////////////////////////////////////////
// Includes
///////////////////////////////////////    
#pragma mark Includes

// Apple stuff
#include <Carbon/Carbon.h>
#include <CoreFoundation/CoreFoundation.h>
#include <Security/Authorization.h>

// Unix stuff
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>


////////////////////////////////////////
// Definitions
///////////////////////////////////////    
#pragma mark Definitions

// name length limits
#define	kMaxInterpreterLength	256

#define	kDefaultPathLength	1024

// execution modes
#define	kErrGettingExecStyle	0
#define	kNormalExecution	1
#define	kPrivilegedExecution	2

//exec style strings
#define kExecStyleNoAuth	"NoAuth"
#define kExecStyleAuth		"Auth"

// names of files bundled with app
#define	kScriptFileName		"script"
#define	kInterpreterFileName	"interpreter"
#define	kExecStyleFileName	"auth"
    
////////////////////////////////////////
// Prototypes
///////////////////////////////////////    
#pragma mark Prototypes

static OSErr ExecuteScript (char *interpreter, char *script);
static OSErr ExecuteScriptWithPrivileges (char *interpreter, char *script);

static char* GetScript (void);
static char* GetInterpreter (void);
static short GetExecutionStyle (void);

static OSStatus FSMakePath(FSSpec file, char *path, long maxPathSize);
static void RedFatalAlert (Str255 errorString, Str255 expStr);
static short DoesFileExist (char *path);
static char* GetFileNameFromPath (char *name);

////////////////////////////////////////
// globals
///////////////////////////////////////    
#pragma mark Globals

long	maxPathSize;

extern char	**environ;

#pragma mark -

////////////////////////////////////////
// Program entrance point
///////////////////////////////////////
int main(int argc, char* argv[])
{
    OSStatus		err = noErr;
    char		*interpreter;
    char		*scriptPath;
    short		execStyle;

    // get maximum path size -- just to be sure.  This could change in future Darwin updates
    maxPathSize = pathconf( "/", _PC_PATH_MAX);
    if (maxPathSize == -1)
        maxPathSize = kDefaultPathLength;
    
    // Make sure that both 'interpreter', 'execStyle'
    // and 'script' files are present and get their data
    interpreter = GetInterpreter();
    if (interpreter == NULL)
        RedFatalAlert("\pInitialization Error", "\pError getting interpreter from application package.");
    
    execStyle = GetExecutionStyle();
    if (execStyle == kErrGettingExecStyle)
        RedFatalAlert("\pInitialization Error", "\pError determining execution style.");
    
    scriptPath = GetScript();
    if (scriptPath == NULL)
        RedFatalAlert("\pInitialization Error", "\pError getting script from application package.");

    InitCursor();

    // Execute script
    switch(execStyle)
    {
        //don't need administrator privileges
        case kNormalExecution:
        {
            // allocate memory for command string
            //command = malloc(maxPathSize);
            //create command
            //sprintf(command, "'%s' '%s'",  interpreter, scriptPath);
            //free memory allocated for script path string
            //free(scriptPath);
            //execute it
            err = ExecuteScript(interpreter, scriptPath);
            //free memory allocated for command string
           // free(command);
            
        } break;
        
        //we need to authenticate - script runs as root
        case kPrivilegedExecution:
        {
            //authenticate and run
            err = ExecuteScriptWithPrivileges (interpreter, scriptPath);
            //free memory allocated for script path string
            free(scriptPath);
            
        } break;
    }
    
    // Check if execution went smoothly
    if (err)
    {
        RedFatalAlert("\pExecution error", "\pAn error occurred while executing program script");
    }

    return 0;
}


#pragma mark -

////////////////////////////////////////
// Run script via the system command
///////////////////////////////////////
static OSErr ExecuteScript (char *interpreter, char *script)
{
    char	errStr[256];
    Str255	pErrStr;
    pid_t	pid;
    pid_t	wpid;
    int		status;
    char	*arguments[] = { (char *)GetFileNameFromPath(interpreter), script, NULL };
    
    //open fork
    pid = fork();
    
    if (pid == (pid_t)-1)
    {
        //error
        sprintf((char *)&errStr, "Forking failed.  fork() returned %d", pid);
        CopyCStringToPascal ((char *)&errStr, pErrStr);
        RedFatalAlert("\pExecution Error", pErrStr);
        exit(13);
    }
    else if (pid == 0)
    {
        //child process started
        //execute the script
        execve(interpreter, arguments, environ);
        
        //if we reach this point there's an error
        sprintf((char *)&errStr, "%s %s execve failed.", interpreter, arguments[1]);
        CopyCStringToPascal ((char *)&errStr, pErrStr);
        RedFatalAlert("\pExecution Error", pErrStr);
    }
    
    //wait while child process finishes
    wpid = wait(&status);
    if (wpid == (pid_t)-1)
    {	
        //ERROR
        RedFatalAlert("\pExecution Error", "\pPID error when wait()ing on child process.");
        return wpid;  
    }
    
    return noErr;
}

////////////////////////////////////////
// Authenticate and run script as root
///////////////////////////////////////
static OSErr ExecuteScriptWithPrivileges (char *interpreter, char *script)
{
    OSErr			err = noErr;
    AuthorizationRef 		authorizationRef;
    char			*arguments[2] = { script, NULL };
    long			authFlags = kAuthorizationFlagDefaults;
    
    // Use Apple's Authentication Manager APIs to run script with root privileges
    err = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, &authorizationRef);
    if (err != errAuthorizationSuccess)
        return err;

    err = AuthorizationExecuteWithPrivileges(authorizationRef, interpreter, authFlags, arguments, NULL);
    if (err < 0)//some kind of error took place
        return true;

    return noErr;
}



#pragma mark -

////////////////////////////////////////
// Get path to the script in Resources folder
///////////////////////////////////////
static char* GetScript (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		scriptFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    char		*path;
    OSErr		err = noErr;

    //get CF URL for interpreter text file
    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, kScriptFileName, kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    scriptFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (scriptFileURL == NULL)
        return NULL;
    
    //Get file reference from Core Foundation URL
    if ( !CFURLGetFSRef( scriptFileURL, &fileRef ))
        return NULL;
    
    //dispose of the CF variables
    CFRelease( scriptFileURL );
    CFRelease( fileName );
    
    //convert FSRef to FSSpec
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
        
    //create path string
    path = malloc(maxPathSize);
    if (path == NULL)
        return NULL;
    
    //convert fsspec to path string
    err = FSMakePath(fileSpec, path, maxPathSize);
    if (err)
        return NULL;
    
    //make sure it exists :)
    if (!DoesFileExist(path))
        return NULL;
    
    return (path);

}

////////////////////////////////////////
// Gets the path to interpreter from
// text file in resources folder
///////////////////////////////////////
static char* GetInterpreter (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		interpreterFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    static char 	interpreter[kMaxInterpreterLength];
    char		*path;
    FILE		*file;
    OSErr		err;
    
    //get CF URL for interpreter text file
    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, kInterpreterFileName, kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    interpreterFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (interpreterFileURL == NULL)
        return NULL;
    
    //Get file reference from Core Foundation URL
    if ( !CFURLGetFSRef( interpreterFileURL, &fileRef ))
        return NULL;
    
    //dispose of the CF variables
    CFRelease( interpreterFileURL );
    CFRelease( fileName );
        
    //convert FSRef to FSSpec
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
    
    //create path string
    path = malloc(maxPathSize);
    if (path == NULL)
        return NULL;
    
    //convert fsspec to path string
    err = FSMakePath(fileSpec, path, maxPathSize);
    if (err)
        return NULL;
    
    //make sure text file containing interpreter exists
    if (!DoesFileExist(path))
        return NULL;
    
    //open file, read interpreter from first line, close
    file = fopen(path, "r");
    fgets(interpreter, kMaxInterpreterLength, file);
    fclose(file);
    
    //free memory containing path name
    free(path);

    //make sure interpreter exists
    if (!DoesFileExist((char *)&interpreter))
        RedFatalAlert("\pExecution Error", "\pThe interpreter for the script this application executes could not be found.");
    
    return ((char *)&interpreter);
}

////////////////////////////////////////
// Open text file in Resources folder
// which tells whether the script needs
// to run as root or not
///////////////////////////////////////
static short GetExecutionStyle (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		execStyleFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    char 		str[7];
    char		*path;
    FILE		*file;
    OSErr		err;
    
    //get CF URL for interpreter text file
    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, kExecStyleFileName, kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    execStyleFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (execStyleFileURL == NULL)
        return NULL;
    
    //Get file reference from Core Foundation URL
    if ( !CFURLGetFSRef( execStyleFileURL, &fileRef ))
        return NULL;
    
    //dispose of the CF variables
    CFRelease( execStyleFileURL );
    CFRelease( fileName );
        
    //convert FSRef to FSSpec
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
    
    //create path string
    path = malloc(maxPathSize);
    if (path == NULL)
        return NULL; 
        
    //convert fsspec to path string
    err = FSMakePath(fileSpec, path, maxPathSize);
    if (err)
        return NULL;
    
    //make sure text file containing interpreter exists
    if (!DoesFileExist(path))
        return NULL;
    
    //open file, read exec style from first line, close
    file = fopen(path, "r");
    fgets(str,7,file);
    fclose(file);
    
    //free memory containing path string
    free(path);

    //return style based on what the exec style text file contains
    if (!strcmp((char *)&str, (char *)&kExecStyleNoAuth))
        return kNormalExecution;
    
    if (!strcmp((char *)&str, (char *)&kExecStyleAuth))
        return kPrivilegedExecution;
    
    return kNormalExecution;
}


#pragma mark -


////////////////////////////////////////
// Generate path string from FSSpec record
///////////////////////////////////////
static OSStatus FSMakePath(FSSpec file, char *path, long maxPathSize)
{
    OSErr 	err = noErr;
    FSRef  	fileRef;

    //create file reference from file spec
    err = FSpMakeFSRef(&file, &fileRef);
    if (err)
        return err;

    /* and then convert the FSRef to a path */
    err = FSRefMakePath(&fileRef, path, maxPathSize);
    return (err);
}



////////////////////////////////////////
//Standard red error alert, then exit application
////////////////////////////////////////
static void RedFatalAlert (Str255 errorString, Str255 expStr)
{
    StandardAlert (kAlertStopAlert, errorString,  expStr, NULL, NULL);
    ExitToShell();
}

////////////////////////////////////////
// Determines whether file exists at path or not
///////////////////////////////////////

static short DoesFileExist (char *path)
{
    if (access(path, F_OK) == -1)
            return false;

    return true;	
}

////////////////////////////////////////
// Cuts suffix from a file name
///////////////////////////////////////

static char* GetFileNameFromPath (char *name)
{
    short	i, len;
    
    len = strlen(name);
    
    for (i = len; i > 0; i--)
    {
        if (name[i] == '/')
        {
            return((char *)&name[i+1]);
        }
    }
    return name;
}
