////////////////////////////////////////
// Includes
///////////////////////////////////////    
#pragma mark Includes

// Apple stuff
#include <Carbon/Carbon.h>
#include <CoreFoundation/CoreFoundation.h>
#include <Security/Authorization.h>

// Unix stuff
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include <pthread.h>




////////////////////////////////////////
// Definitions
///////////////////////////////////////    
#pragma mark Definitions

// name length limits
#define	kMaxInterpreterLength	256

#define	kMaxPathLength	1024

// execution modes
#define	kErrGettingExecStyle	0
#define	kNormalExecution	1
#define	kPrivilegedExecution	2

// output modes
#define	kErrGettingOutputType	0
#define	kNoOutput		1
#define	kProgressBarOutput	2
#define	kTextOutput		3

//exec style strings
#define kExecStyleNoAuth	"NoAuth"
#define kExecStyleAuth		"Auth"

//output type strings
#define	kOutputTypeNone		"None"
#define	kOutputTypeProgressBar	"ProgressBar"
#define	kOutputTypeText		"Text"

// names of files bundled with app
#define	kScriptFileName		"script"
#define	kInterpreterFileName	"interpreter"
#define	kExecStyleFileName	"auth"
#define	kOutputTypeFileName	"outputType"
    
    
    
////////////////////////////////////////
// Prototypes
///////////////////////////////////////    
#pragma mark Prototypes


static void *Execute (void *arg);
static OSErr ExecuteScript (char *interpreter, char *script);
static OSErr ExecuteScriptWithPrivileges (char *interpreter, char *script);

static void GetParameters (void);
static char* GetScript (void);
static char* GetInterpreter (void);
static short GetExecutionStyle (void);
static short GetOutputType (void);

OSErr LoadProgressWindow (void);
OSErr LoadMenuBar (char *appName);

static OSStatus FSMakePath(FSSpec file, char *path, long maxPathSize);
static void RedFatalAlert (Str255 errorString, Str255 expStr);
static short DoesFileExist (char *path);
static char* GetFileNameFromPath (char *name);

static OSErr QuitAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon);
void QuitApp (void);



////////////////////////////////////////
// globals
///////////////////////////////////////    
#pragma mark Globals

int	pid = 0;
short	taskDone = true;

    // execution parameters
    char		interpreter[kMaxPathLength];
    char		scriptPath[kMaxPathLength];
    short		execStyle = 0;
    short		outputType = 0;

extern char	**environ;
WindowRef	theWindow;

#pragma mark -

////////////////////////////////////////
// Program entrance point
///////////////////////////////////////
int main(int argc, char* argv[])
{
    OSErr			err = noErr;
    pthread_t 			tid;
    int				ret;
    
    //install Apple Event Quit handler
    AEInstallEventHandler( kCoreEventClass, kAEQuitApplication, NewAEEventHandlerUPP((AEEventHandlerProcPtr)QuitAEHandler), 0, false );

    InitCursor();

    //create the menu bar
    err = LoadMenuBar(NULL);
    if (err != noErr)
        RedFatalAlert("\pInitialization Error", "\pError loading Menu Bar from Nib.");
    
    //load data from files containing exec settings
    GetParameters();
    
    //at this point, we've loaded all the requisite information into globals
    if (outputType == kProgressBarOutput)
    {
        err = LoadProgressWindow();
        if (err != noErr)
            RedFatalAlert("\pInitialization Error", "\pError loading Progress Bar Window from Nib.");
    }

    //open thread which forks and executes script
    ret = pthread_create(&tid, NULL, Execute, NULL);

    // Run the event loop
    RunApplicationEventLoop();

    return 0;
}

#pragma mark -

///////////////////////////////////
// Execution thread starts here
// Determine execution style and
// run the bundled script.
///////////////////////////////////
static void *Execute (void *arg)
{
    OSErr		err = noErr;

    //ok, we've starting executing task and we're not finished
    taskDone = false;
    
    // Execute script
    switch(execStyle)
    {
        //don't need administrator privileges
        case kNormalExecution:
        {
            //execute by forking and execing
            err = ExecuteScript(interpreter, scriptPath);
            
        } break;
        
        //we need to authenticate - script runs as root
        case kPrivilegedExecution:
        {
            //authenticate and run
            err = ExecuteScriptWithPrivileges (interpreter, scriptPath);
            
        } break;
    }
    
    taskDone = true;
    
    // Check if execution went smoothly
    if (err)
    {
        RedFatalAlert("\pExecution error", "\pAn error occurred while executing program script");
    }

    ExitToShell();
    return 0;
}



////////////////////////////////////////
// Run script via the system command
///////////////////////////////////////
static OSErr ExecuteScript (char *interpreter, char *script)
{
    char	errStr[256];
    Str255	pErrStr;
    pid_t	wpid;
    int		status;
    char	*arguments[] = { (char *)GetFileNameFromPath(interpreter), script, NULL };
    
    
    //open fork
    pid = fork();
    
    if (pid == (pid_t)-1)
    {
        //error
        sprintf((char *)&errStr, "Forking failed.  fork() returned %d", pid);
        CopyCStringToPascal ((char *)&errStr, pErrStr);
        RedFatalAlert("\pExecution Error", pErrStr);
        exit(13);
    }
    else if (pid == 0)
    {
        //child process started
        //execute the script
        execve(interpreter, arguments, environ);
        
        //if we reach this point there's an error
        sprintf((char *)&errStr, "%s %s execve failed.", interpreter, arguments[1]);
        CopyCStringToPascal ((char *)&errStr, pErrStr);
        RedFatalAlert("\pExecution Error", pErrStr);
    }

    //wait while child process finishes
    wpid = wait(&status);
    
    
    if (wpid == (pid_t)-1)
    {	
        //ERROR
        RedFatalAlert("\pExecution Error", "\pPID error when wait()ing on child process.");
        return wpid;  
    }
    
    return noErr;
}

////////////////////////////////////////
// Authenticate and run script as root
///////////////////////////////////////
static OSErr ExecuteScriptWithPrivileges (char *interpreter, char *script)
{
    OSErr			err = noErr;
    AuthorizationRef 		authorizationRef;
    char			*arguments[2] = { script, NULL };
    
    // Use Apple's Authentication Manager APIs to run script with root privileges
    err = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, &authorizationRef);
    if (err != errAuthorizationSuccess)
        return err;

    if ((err = AuthorizationExecuteWithPrivileges(authorizationRef, interpreter, kAuthorizationFlagDefaults, arguments, NULL)) != noErr)
    {
        //some kind of error took place
        taskDone = true;
        return true;
    }
    else
    {
            //wait for child process to finish
            int child;
            wait(&child);
    }

    return noErr;
}



#pragma mark -


static void GetParameters (void)
{
    // Make sure that 'interpreter', 'execStyle', 'outputType'
    // and 'script' files are present and get their data
    
    //get path to interpreter
    strcpy((char *)&interpreter, (char *)GetInterpreter());
    if (interpreter == NULL)
        RedFatalAlert("\pInitialization Error", "\pError getting interpreter from application package.");
    
    //get path to script to be executed
    strcpy((char *)&scriptPath, (char *)GetScript());
    if (scriptPath == NULL)
        RedFatalAlert("\pInitialization Error", "\pError getting script from application package.");
    
    //check if we need root privileges
    execStyle = GetExecutionStyle();
    if (execStyle == kErrGettingExecStyle)
        RedFatalAlert("\pInitialization Error", "\pError determining execution style.");
    
    //check if we're doing some kind of output
    outputType = GetOutputType();
    if (outputType == kErrGettingOutputType)
        RedFatalAlert("\pInitialization Error", "\pError determining execution style.");


}


////////////////////////////////////////
// Get path to the script in Resources folder
///////////////////////////////////////
static char* GetScript (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		scriptFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    char		*path;
    OSErr		err = noErr;

    //get CF URL for interpreter text file
    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, kScriptFileName, kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    scriptFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (scriptFileURL == NULL)
        return NULL;
    
    //Get file reference from Core Foundation URL
    if ( !CFURLGetFSRef( scriptFileURL, &fileRef ))
        return NULL;
    
    //dispose of the CF variables
    CFRelease( scriptFileURL );
    CFRelease( fileName );
    
    //convert FSRef to FSSpec
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
        
    //create path string
    path = malloc(kMaxPathLength);
    if (path == NULL)
        return NULL;
    
    //convert fsspec to path string
    err = FSMakePath(fileSpec, path, kMaxPathLength);
    if (err)
        return NULL;
    
    //make sure it exists :)
    if (!DoesFileExist(path))
        return NULL;
    
    return (path);

}

////////////////////////////////////////
// Gets the path to interpreter from
// text file in resources folder
///////////////////////////////////////
static char* GetInterpreter (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		interpreterFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    static char 	interpreter[kMaxInterpreterLength];
    char		*path;
    FILE		*file;
    OSErr		err;
    
    //get CF URL for interpreter text file
    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, kInterpreterFileName, kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    interpreterFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (interpreterFileURL == NULL)
        return NULL;
    
    //Get file reference from Core Foundation URL
    if ( !CFURLGetFSRef( interpreterFileURL, &fileRef ))
        return NULL;
    
    //dispose of the CF variables
    CFRelease( interpreterFileURL );
    CFRelease( fileName );
        
    //convert FSRef to FSSpec
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
    
    //create path string
    path = malloc(kMaxPathLength);
    if (path == NULL)
        return NULL;
    
    //convert fsspec to path string
    err = FSMakePath(fileSpec, path, kMaxPathLength);
    if (err)
        return NULL;
    
    //make sure text file containing interpreter exists
    if (!DoesFileExist(path))
        return NULL;
    
    //open file, read interpreter from first line, close
    file = fopen(path, "r");
    fgets(interpreter, kMaxInterpreterLength, file);
    fclose(file);
    
    //free memory containing path name
    free(path);

    //make sure interpreter exists
    if (!DoesFileExist((char *)&interpreter))
        RedFatalAlert("\pExecution Error", "\pThe interpreter for the script this application executes could not be found.");
    
    return ((char *)&interpreter);
}

////////////////////////////////////////
// Open text file in Resources folder
// which tells whether the script needs
// to run as root or not
///////////////////////////////////////
static short GetExecutionStyle (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		execStyleFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    char 		str[7];
    char		*path;
    FILE		*file;
    OSErr		err;
    
    //get CF URL for interpreter text file
    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, kExecStyleFileName, kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    execStyleFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (execStyleFileURL == NULL)
        return NULL;
    
    //Get file reference from Core Foundation URL
    if ( !CFURLGetFSRef( execStyleFileURL, &fileRef ))
        return NULL;
    
    //dispose of the CF variables
    CFRelease( execStyleFileURL );
    CFRelease( fileName );
        
    //convert FSRef to FSSpec
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
    
    //create path string
    path = malloc(kMaxPathLength);
    if (path == NULL)
        return NULL; 
        
    //convert fsspec to path string
    err = FSMakePath(fileSpec, path, kMaxPathLength);
    if (err)
        return NULL;
    
    //make sure text file containing interpreter exists
    if (!DoesFileExist(path))
        return NULL;
    
    //open file, read exec style from first line, close
    file = fopen(path, "r");
    fgets(str,7,file);
    fclose(file);
    
    //free memory containing path string
    free(path);

    //return style based on what the exec style text file contains
    if (!strcmp((char *)&str, (char *)&kExecStyleNoAuth))
        return kNormalExecution;
    
    if (!strcmp((char *)&str, (char *)&kExecStyleAuth))
        return kPrivilegedExecution;
    
    return kNormalExecution;
}


////////////////////////////////////////
// Open text file in Resources folder that
// contains output type, i.e. progress bar, text etc.
///////////////////////////////////////
static short GetOutputType (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		outputTypeFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    char 		str[12];
    char		*path;
    FILE		*file;
    OSErr		err;
    
    //get CF URL for interpreter text file
    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, kOutputTypeFileName, kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    outputTypeFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (outputTypeFileURL == NULL)
        return NULL;
    
    //Get file reference from Core Foundation URL
    if ( !CFURLGetFSRef( outputTypeFileURL, &fileRef ))
        return NULL;
    
    //dispose of the CF variables
    CFRelease( outputTypeFileURL );
    CFRelease( fileName );
        
    //convert FSRef to FSSpec
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
    
    //create path string
    path = malloc(kMaxPathLength);
    if (path == NULL)
        return NULL; 
        
    //convert fsspec to path string
    err = FSMakePath(fileSpec, path, kMaxPathLength);
    if (err)
        return NULL;
    
    //make sure text file containing interpreter exists
    if (!DoesFileExist(path))
        return NULL;
    
    //open file, read exec style from first line, close
    file = fopen(path, "r");
    fgets(str,sizeof(str),file);
    fclose(file);
    
    //free memory containing path string
    free(path);

    //return style based on what the exec style text file contains
    if (!strcmp((char *)&str, (char *)&kOutputTypeNone))
        return kNoOutput;
    
    if (!strcmp((char *)&str, (char *)&kOutputTypeProgressBar))
        return kProgressBarOutput;
    
    if (!strcmp((char *)&str, (char *)&kOutputTypeText))
        return kTextOutput;
    
    return kNoOutput;
}


#pragma mark -

//////////////////////////////////////
//  Load modal progress bar
/////////////////////////////////////

OSErr LoadProgressWindow (void)
{
    OSErr		err;
    IBNibRef 		nibRef;
    
    err = CreateNibReference(CFSTR("ProgressWindow"), &nibRef);
    if (err)
        return err;
        
    err = CreateWindowFromNib(nibRef, CFSTR("Window"), &theWindow);
    if (err)
        return err;

    DisposeNibReference(nibRef);
    
    // The window was created hidden so show it.
    ShowWindow( theWindow );

    return noErr;
}

//////////////////////////////////////
//  Load menu bar
/////////////////////////////////////

OSErr LoadMenuBar (char *appName)
{
    OSErr		err;
    IBNibRef 		nibRef;
    
    err = CreateNibReference(CFSTR("MenuBar"), &nibRef);
    if (err)
        return err;
    
    err = SetMenuBarFromNib(nibRef, CFSTR("MenuBar"));
    if (err)
        return err;

    DisposeNibReference(nibRef);

    return noErr;
}



#pragma mark -

////////////////////////////////////////
// Generate path string from FSSpec record
///////////////////////////////////////
static OSStatus FSMakePath(FSSpec file, char *path, long maxPathSize)
{
    OSErr 	err = noErr;
    FSRef  	fileRef;

    //create file reference from file spec
    err = FSpMakeFSRef(&file, &fileRef);
    if (err)
        return err;

    /* and then convert the FSRef to a path */
    err = FSRefMakePath(&fileRef, path, maxPathSize);
    return (err);
}



////////////////////////////////////////
//Standard red error alert, then exit application
////////////////////////////////////////
static void RedFatalAlert (Str255 errorString, Str255 expStr)
{
    StandardAlert (kAlertStopAlert, errorString,  expStr, NULL, NULL);
    ExitToShell();
}

////////////////////////////////////////
// Determines whether file exists at path or not
///////////////////////////////////////

static short DoesFileExist (char *path)
{
    if (access(path, F_OK) == -1)
            return false;

    return true;	
}

////////////////////////////////////////
// Cuts suffix from a file name
///////////////////////////////////////

static char* GetFileNameFromPath (char *name)
{
    short	i, len;
    
    len = strlen(name);
    
    for (i = len; i > 0; i--)
    {
        if (name[i] == '/')
        {
            return((char *)&name[i+1]);
        }
    }
    return name;
}

#pragma mark -

////////////////////////////////////////
// Apple Event handler for Quit i.e. from
// the dock or Application menu item
///////////////////////////////////////
static OSErr QuitAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon)
{
	#pragma unused (reply, refCon, theAppleEvent)
	
  	QuitApp();
  	
  	return noErr;
}

////////////////////////////////////////
// Do needed stuff and then exit app
///////////////////////////////////////
void QuitApp (void)
{
    if (taskDone != true)
    {
        kill(pid, SIGTERM);
        
    }
    printf("%d killed\n", pid);
    ExitToShell();
}


