#import "Platypus.h"
#include <stdio.h>

@implementation Platypus


- (id)init
{
	if (self = [super init]) 
	{
		//create file list
		fileList = [[FileList alloc] init];
		typesList = [[TypesList alloc] init];
		suffixList = [[SuffixList alloc] init];
		recentItems = [[NSMutableArray alloc] initWithCapacity: 10];
    }
    return self;
}

-(void)dealloc
{
    [fileList release];
	[typesList release];
	[suffixList release];
	[recentItems release];
    [super dealloc];
}

+ (void)initialize 
{ 
	// create the user defaults here if none exists
    // create a dictionary
    NSMutableDictionary *defaultPrefs = [NSMutableDictionary dictionary];
    // put default prefs in the dictionary
	[defaultPrefs setObject: [NSNumber numberWithBool:YES] forKey:@"ParseShebangLineForInterpreter"];
	[defaultPrefs setObject: [NSNumber numberWithBool:NO]  forKey:@"RevealApplicationWhenCreated"];
	[defaultPrefs setObject: @"TextEdit"  forKey:@"DefaultEditor"];
	[defaultPrefs setObject: [NSNumber numberWithBool:NO] forKey:@"ShowAdvancedOptions"];
	[defaultPrefs setObject: [NSArray array] forKey:@"Profiles"];
	
    // register the dictionary of defaults
    [[NSUserDefaults standardUserDefaults] registerDefaults: defaultPrefs];
}

/*****************************************
 - Handler for when app is done launching
 - Set up the window and stuff like that
*****************************************/
- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	//set up window
	[window center];
	if ([[NSUserDefaults standardUserDefaults] boolForKey:@"ShowAdvancedOptions"] == YES)
	{
		[showAdvancedArrow setState: YES];
		[self toggleAdvancedOptions: self];
	}
	[window registerForDraggedTypes: [NSArray arrayWithObjects:NSFilenamesPboardType, nil]];
	[window makeKeyAndOrderFront: NULL];

	//load recent items
	[recentItems addObjectsFromArray: [[NSUserDefaults standardUserDefaults] objectForKey:@"RecentItems"]];
	[self constructOpenRecentMenu];
	
	//load profiles
	[self constructProfilesMenu];
	
	//register the data source for the types and suffix lists
	[typesWindow center];
	[self setDefaultTypes: self];
	[typesListDataBrowser setDataSource: typesList];
	[typesListDataBrowser reloadData];
	[suffixListDataBrowser setDataSource: suffixList];
	[suffixListDataBrowser reloadData];
	
	//set icons for editor menu
	[self setIconsForEditorMenu];
}

/*****************************************
 - Handler for application termination
*****************************************/
- (void)applicationWillTerminate:(NSNotification *)aNotification
{
	//save recent items
	[[NSUserDefaults standardUserDefaults] setObject: recentItems  forKey:@"RecentItems"];
	
	//save window status
	[[NSUserDefaults standardUserDefaults] setBool: [showAdvancedArrow state]  forKey:@"ShowAdvancedOptions"];
	
}

/*****************************************
 - Handler for dragged files and/or files opened via the Finder
*****************************************/

- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
{
	BOOL	isDir = NO;

	if([[NSFileManager defaultManager] fileExistsAtPath: filename isDirectory: &isDir] && !isDir)
	{
		[self loadScript: filename];
		return(YES);
	}
	else
		return(NO);
}

#pragma mark -


/*****************************************
 - Create a new script and open in external editor
*****************************************/

- (IBAction)newScript:(id)sender
{
	int			i = 0;
	NSString	*tempScript = @"/tmp/PlatypusScript";
	NSArray *defaultInterpreters = [NSArray arrayWithObjects: @"/bin/sh",@"/usr/bin/python",@"/usr/bin/osascript",@"/usr/bin/expect",@"/usr/bin/perl",@"/usr/bin/ruby",@"/usr/bin/tclsh",@"/usr/bin/php", nil];
	
	//put shebang line in the new script text file
	NSString	*shebangStr = [NSString stringWithFormat: @"#!%@\n\n", [defaultInterpreters objectAtIndex: [[scriptTypeRadioButtons selectedCell] tag]]];
	
	//if this is a perl or shell script, we add a commented list of paths to the bundled files 
	if (([[scriptTypeRadioButtons selectedCell] tag] == 0 || [[scriptTypeRadioButtons selectedCell] tag] == 4) && [fileList numFiles] > 0)
	{
		shebangStr = [shebangStr stringByAppendingString: @"# You can access your bundled files at the following paths:\n#\n"];
		for (i = 0; i < [fileList numFiles]; i++)
		{
			if ([[scriptTypeRadioButtons selectedCell] tag] == 0)//shell script
				shebangStr = [shebangStr stringByAppendingString: [NSString stringWithFormat:@"# \"$1/Contents/Resources/%@\"\n", [fileList getFileAtIndex: i]]];
			else if ([[scriptTypeRadioButtons selectedCell] tag] == 4)//perl script
				shebangStr = [shebangStr stringByAppendingString: [NSString stringWithFormat:@"# \"$ARGV[0]/Contents/Resources/%@\"\n", [fileList getFileAtIndex: i]]];
		}
		shebangStr = [shebangStr stringByAppendingString: @"#\n#\n\n"];
	}
	
	[shebangStr writeToFile: tempScript atomically: YES];

	[self loadScript: tempScript];
	[self editScript: self];

}

/*****************************************
 - Reveal script in Finder
*****************************************/

- (IBAction)revealScript:(id)sender
{
	if ([[scriptPathTextField stringValue] length] == 0)
		return;
		
	//see if file exists
	if ([[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] ])
	{
		[[NSWorkspace sharedWorkspace] selectFile:[scriptPathTextField stringValue] inFileViewerRootedAtPath:nil];
	}
	else
	{
		[STUtil alert:@"File does not exist" subText: @"No file exists at the specified path"];
	}
}


/*****************************************
 - Open script in external editor
*****************************************/

- (IBAction)editScript:(id)sender
{
	if ([[scriptPathTextField stringValue] length] == 0)
		return;
		
	//see if file exists
	if ([[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] ])
	{
		NSArray *args = [NSArray arrayWithObjects: @"-a", [[NSUserDefaults standardUserDefaults] stringForKey:@"DefaultEditor"], [scriptPathTextField stringValue], nil];
		[NSTask launchedTaskWithLaunchPath:@"/usr/bin/open" arguments: args];
	}
	else
		[STUtil alert:@"File does not exist" subText: @"No file exists at the specified path"];
}


/*****************************************
 - Run the script in Terminal.app
*****************************************/
- (IBAction)runScript:(id)sender
{
	NSTask	*theTask = [[NSTask alloc] init];
	//the applescript command to run the script in Terminal.app
	NSString *osaCmd = [NSString stringWithFormat: @"tell application \"Terminal\"\n\tdo script \"%@ %@\"\nend tell", [interpreterTextField stringValue], [scriptPathTextField stringValue]];
	
	//initialize task
	[theTask setLaunchPath: @"/usr/bin/osascript"];
	[theTask setArguments: [NSArray arrayWithObjects: @"-e", osaCmd, nil]];
	[theTask setStandardError: [NSFileHandle fileHandleWithNullDevice]];
	[theTask setStandardOutput: [NSFileHandle fileHandleWithNullDevice]];
	
	//launch, wait until it's done and then release it
	[theTask launch];
	[theTask waitUntilExit];
	[theTask release];
}

#pragma mark -

/*****************************************
 - Create button was pressed: Verify that field values are valid
 - Then put up a sheet for designating location to create application
*****************************************/

- (IBAction)createButtonPressed: (id)sender
{
	if (![self verifyFieldContents])//are there invalid values in the fields?
		return;

	NSSavePanel *sPanel = [NSSavePanel savePanel];
	[sPanel setPrompt:@"Create"];
	[window setTitle: @"Platypus - Select place to create app"];
	
	//run open panel
    [sPanel beginSheetForDirectory:nil file:[appNameTextField stringValue] modalForWindow: window modalDelegate: self didEndSelector: @selector(createApp:returnCode:contextInfo:) contextInfo: nil];

}

/*****************************************
 - generate application bundle from data provided
*****************************************/
- (void)createApp:(NSSavePanel *)sPanel returnCode:(int)result contextInfo:(void *)contextInfo
{
	int	      i;
	NSString *appPath, *contentsPath, *macosPath, *resourcesPath, *lprojPath;
	NSString *execDestinationPath, *infoPlistPath, *iconPath, *bundledFileDestPath, *progressBarNibDestPath;
	NSString *execPath, *bundledFilePath, *progressBarNibPath;
	NSString *scriptFilePath, *pkgInfoPath, *appSettingsPlistPath;
	NSString *appName;
	NSString *infoPlistStrings;
	NSBundle *platypusBundle = [NSBundle mainBundle];
	NSMutableDictionary	*appSettingsPlist;
	NSFileManager *fileManager = [NSFileManager defaultManager];
	

	/////////////**************init and run save panel**********************//////////////
	if (result == NSOKButton) 
	{
		appPath = [sPanel filename];
		//check if app already exists, and if so, prompt if to replace
		if ([fileManager fileExistsAtPath: [appPath stringByAppendingString:@".app"]])
		{
			if (NO == [STUtil proceedWarning: @"Application already exists" subText: @"An application with this name already exists in the location you specified.  Do you wish to overwrite it?"])
				return;
			else
				[fileManager removeFileAtPath: [appPath stringByAppendingString:@".app"] handler: NULL];
		}
	}
	else
		return;
	
	
	
	//////////********* Create the folder hierarchy ************////////////////
	
			//Application.app bundle
			if (![appPath hasSuffix:@".app"])
			{
				appPath = [appPath stringByAppendingString:@".app"];
			}
			[fileManager createDirectoryAtPath: appPath attributes:nil];
			
			//Contents
			contentsPath = [appPath stringByAppendingString:@"/Contents"];
			[fileManager createDirectoryAtPath: contentsPath attributes:nil];
			
			//MacOS
			macosPath = [contentsPath stringByAppendingString:@"/MacOS"];
			[fileManager createDirectoryAtPath: macosPath attributes:nil];
			
			//Resources
			resourcesPath = [contentsPath stringByAppendingString:@"/Resources"];
			[fileManager createDirectoryAtPath: resourcesPath attributes:nil];
			
			//English.lproj
			lprojPath = [resourcesPath stringByAppendingString:@"/English.lproj"];
			[fileManager createDirectoryAtPath: lprojPath attributes:nil];
			
			//PkgInfo file
			pkgInfoPath = [contentsPath stringByAppendingString:@"/PkgInfo"];
			[[NSString stringWithString:@"APPL????"] writeToFile: pkgInfoPath atomically: YES];
	
	//////////********* Copy files to app bundle ************////////////////
	
		//copy exec file
		appName = [STUtil cutSuffix: [appPath lastPathComponent]];
		execPath = [platypusBundle pathForResource:@"exec" ofType:nil];
		execDestinationPath = [macosPath stringByAppendingString:@"/"];
		execDestinationPath = [execDestinationPath stringByAppendingString: appName]; 
		[fileManager copyPath:execPath toPath:execDestinationPath handler:nil];
		
			//copy nib file to app bundle
			progressBarNibPath = [platypusBundle pathForResource:@"MainMenu.nib" ofType:nil];
			progressBarNibDestPath = [lprojPath stringByAppendingString:@"/MainMenu.nib"];
			[fileManager copyPath:progressBarNibPath toPath:progressBarNibDestPath handler:nil];
			
			//create InfoPlist.strings
			infoPlistStrings = [NSString stringWithFormat:
								@"CFBundleName = \"%@\";\nCFBundleShortVersionString = \"%@\";\nCFBundleGetInfoString = \"%@ version %@ Copyright %d %@\";\nNSHumanReadableCopyright = \"Copyright %d %@.\";",  appName, [versionTextField stringValue], appName, [versionTextField stringValue], [[NSCalendarDate calendarDate] yearOfCommonEra], NSFullUserName(), [[NSCalendarDate calendarDate] yearOfCommonEra], NSFullUserName()];
			[infoPlistStrings writeToFile:  [lprojPath stringByAppendingString:@"/InfoPlist.strings"] atomically: YES];

		//create AppSettings.plist file
		appSettingsPlist = [NSMutableDictionary dictionaryWithCapacity: 40];
		[appSettingsPlist setObject: [NSNumber numberWithBool:[rootPrivilegesCheckbox state]] forKey:@"RequiresAdminPrivileges"];
		[appSettingsPlist setObject: [NSNumber numberWithBool:[isDroppableCheckbox state]] forKey:@"IsDroppable"];
		[appSettingsPlist setObject: [NSNumber numberWithBool:[remainRunningCheckbox state]] forKey:@"RemainRunningAfterCompletion"];
		[appSettingsPlist setObject: [[outputTypeRadioButtons selectedCell] title] forKey:@"OutputType"];
		[appSettingsPlist setObject: [interpreterTextField stringValue] forKey:@"ScriptInterpreter"];
		appSettingsPlistPath = [resourcesPath stringByAppendingString:@"/AppSettings.plist"];
		[appSettingsPlist writeToFile: appSettingsPlistPath atomically: YES];
		
		// copy script file over to app bundle
		scriptFilePath = [resourcesPath stringByAppendingString:@"/script"];
		[fileManager copyPath: [scriptPathTextField stringValue] toPath: scriptFilePath handler:nil];
				
		//create icon
		iconPath = [resourcesPath stringByAppendingString:@"/appIcon.icns"];
		[self writeImageAsIcon: [appIconImageWell image] path: iconPath];
		
		//create Info.plist file
		infoPlistPath = [contentsPath stringByAppendingString:@"/Info.plist"];
		NSDictionary *infoPlist = [self getInfoPropertyListDictionary: appName isDroppable: [isDroppableCheckbox state] inBackground: [showInDockCheckbox state] ];
		[infoPlist writeToFile: infoPlistPath atomically: YES];
		
		//copy files in file list to the Resources folder
		for (i = 0; i < [fileList numFiles]; i++)
		{
			bundledFilePath = [fileList getFileAtIndex: i];
			bundledFileDestPath = [resourcesPath stringByAppendingString:@"/"];
			bundledFileDestPath = [bundledFileDestPath stringByAppendingString: [bundledFilePath lastPathComponent]];
			[fileManager copyPath: bundledFilePath toPath: bundledFileDestPath handler:nil];
		}
		
		//reveal app in Finder, if prefs say so
		if ([[NSUserDefaults standardUserDefaults] boolForKey:@"RevealApplicationWhenCreated"] == YES)
		{
			BOOL		isDir;
			
			if ([[NSFileManager defaultManager] fileExistsAtPath:appPath isDirectory:&isDir]) 
			{
						[[NSWorkspace sharedWorkspace] selectFile:appPath inFileViewerRootedAtPath:nil];
			}
		}
		
		//restore window title
		[window setTitle: @"Platypus"];
}


/*****************************************
 - Make sure that all fields contain valid values
*****************************************/

- (BOOL)verifyFieldContents
{
	BOOL			isDir;
	//file manager
	NSFileManager *fileManager = [NSFileManager defaultManager];

	//script path
	if ([[appNameTextField stringValue] length] == 0)//make sure a name has been assigned
	{
		[STUtil sheetAlert:@"Invalid Application Name" subText: @"You must specify a name for your application" forWindow: window];
		return NO;
	}

	//script path
	if (([fileManager fileExistsAtPath: [scriptPathTextField stringValue] isDirectory: &isDir] == NO) || isDir)//make sure script exists and isn't a folder
	{
		[STUtil sheetAlert:@"Invalid Script Path" subText: @"No file exists at the script path you have specified" forWindow: window];
		return NO;
	}
	//interpreter
	if ([fileManager fileExistsAtPath: [interpreterTextField stringValue]] == NO)//make sure interpreter exists
	{
		if (NO == [STUtil proceedWarning: @"Invalid Interpreter" subText: @"The specified interpreter does not exist on this system.  Do you wish to proceed anyway?"])
			return NO;
	}
	//make sure typeslist contains valid values
	if ([typesList numTypes] <= 0 && [isDroppableCheckbox state] == YES)
	{
		[STUtil sheetAlert:@"Invalid Types List" subText: @"The app has been set to be droppable but no file types are set.  Please modify the Types list to correct this." forWindow: window];
		return NO;
	}
	//make sure the signature is 4 characters
	if ([[signatureTextField stringValue] length] != 4)
	{
		[STUtil sheetAlert:@"Invalid App Signature" subText: @"The signature set for the application is invalid.  An app's signature must consist of four upper or lowercase ASCII characters." forWindow: window];
		return NO;
	}
	
	return YES;
}


/*****************************************
 - Called when script type radio button is pressed
*****************************************/

- (IBAction)scriptTypeSelected:(id)sender
{
	[self setScriptType: [[sender selectedCell] tag] ];
}

/*****************************************
 - Updates data in interpreter, icon and script type radio buttons
*****************************************/

- (void)setScriptType: (int)typeNum
{
	NSArray *defaultInterpreters = [NSArray arrayWithObjects: @"/bin/sh",@"/usr/bin/python",@"/usr/bin/osascript",@"/usr/bin/expect",@"/usr/bin/perl",@"/usr/bin/ruby",@"/usr/bin/tclsh",@"/usr/bin/php", nil];
	[interpreterTextField setStringValue: [defaultInterpreters objectAtIndex: typeNum ] ];
	[self setAppIconForType: typeNum];
	[scriptTypeRadioButtons selectCellWithTag: typeNum];
}

/*****************************************
 - Parse the Shebang line (#!) to get the interpreter for the script
*****************************************/

- (NSString *)getInterpreterFromShebang: (NSString *)path
{
	FILE			*file;
	static char		str[1024];
	char			filePath[1024];
	char			len,i;
	NSString		*interpreter;
	
	[path getCString: (char *)&filePath maxLength: sizeof(filePath)];
	
	//open file and read to the first newline character
	file = fopen(filePath, "r");
	fgets(str,1024,file);
	fclose(file);
	
	//doesn't seem to be a shebang line
	if (str[0] != '#' || str[1] != '!')
		return @"";
	
	//jump over the first two characters that indicate the shebang
	strcpy((char *)&str, (char *)&str[2]);
	
	len = strlen((char *)&str);

	for (i = 0; i < len; i++)
	{
		//terminate string when we reach a space or a newline
		if (str[i] == '\n'|| str[i] == ' ')
		{
			str[i] = NULL;
		}
	}
	interpreter = [NSString stringWithCString: (char *)&str];
	
	return interpreter;
}

/*****************************************
 - Each script type has a certain icon associated with it.
 - Set the icon according to the type specified
*****************************************/

- (void)setAppIconForType: (int)type
{
	switch(type)
	{
		case 0:
			[appIconImageWell setImage: [NSImage imageNamed:@"PlatypusDflt"]];
			[iconNameTextField setStringValue:@"Platypus Default"];
			break;
		case 1:
			[appIconImageWell setImage: [NSImage imageNamed:@"PythonSnake"]];
			[iconNameTextField setStringValue:@"Python Snake"];
			break;
		case 2:
			[appIconImageWell setImage: [NSImage imageNamed:@"AppleScript"]];
			[iconNameTextField setStringValue:@"AppleScript"];
			break;
		case 3:
			[appIconImageWell setImage: [NSImage imageNamed:@"ExpectMonkey"]];
			[iconNameTextField setStringValue:@"Hexley"];
			break;
		case 4:
			[appIconImageWell setImage: [NSImage imageNamed:@"PerlCamel"]];
			[iconNameTextField setStringValue:@"Perl Camel"];
			break;
		case 5:
			[appIconImageWell setImage: [NSImage imageNamed:@"RubyGoat"]];
			[iconNameTextField setStringValue:@"Ruby Goat"];
			break;
		case 6:
			[appIconImageWell setImage: [NSImage imageNamed:@"TclFlamingo"]];
			[iconNameTextField setStringValue:@"Tcl Flamingo"];
			break;
		case 7:
			[appIconImageWell setImage: [NSImage imageNamed:@"PHPBird"]];
			[iconNameTextField setStringValue:@"PHP Bird"];
			break;
		case 8:
			[appIconImageWell setImage: [NSImage imageNamed:@"PlatypusDroplet"]];
			[iconNameTextField setStringValue:@"Platypus Droplet"];
			break;
		case 9:
			[appIconImageWell setImage: [NSImage imageNamed:@"PlatypusPlate"]];
			[iconNameTextField setStringValue:@"Platypus Plate"];
			break;
	}
}

/*****************************************
 - Open sheet to select script to load
*****************************************/

- (IBAction)selectScript:(id)sender
{
	//create open panel
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	[oPanel setPrompt:@"Select"];
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setCanChooseDirectories: NO];
	
	[window setTitle: @"Platypus - Select a script"];
	
	//run open panel
    [oPanel beginSheetForDirectory:nil file:nil types:nil modalForWindow: window modalDelegate: self didEndSelector: @selector(selectScriptPanelDidEnd:returnCode:contextInfo:) contextInfo: nil];
		
}

- (void)selectScriptPanelDidEnd:(NSOpenPanel *)oPanel returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if (returnCode == NSOKButton)
		[self loadScript: [oPanel filename]];
	
	[window setTitle: @"Platypus"];
}

/*****************************************
 - Loads script data into platypus window
*****************************************/

- (void)loadScript:(NSString *)filename
{
	NSString	*shebangInterpreter;
	int			i;

	//make sure file we're loading actually exists
	if (![[NSFileManager defaultManager] fileExistsAtPath: filename])
		return;

	//set script path
	[scriptPathTextField setStringValue: filename];

	//set app name
	NSString *appName = [STUtil cutSuffix: [filename lastPathComponent]];
	[appNameTextField setStringValue: appName];
	
	//interpreter
	
	//read shebang line
	//if ([[NSUserDefaults standardUserDefaults] boolForKey:@"ParseShebangLineForInterpreter"] == 1)
	//{
		shebangInterpreter = [self getInterpreterFromShebang: filename];
	/*}
	else
	{
		shebangInterpreter = @"";
	}*/
	
	//if no interpreter can be retrieved from shebang line
	if ([shebangInterpreter caseInsensitiveCompare: @""] == NSOrderedSame)
	{
		//try to determine type from suffix
		[self setScriptType: (int)[self getFileTypeFromSuffix: filename] ];
	}
	else
	{
		//see if interpreter matches a preset
		NSArray *defaultInterpreters = [NSArray arrayWithObjects: @"/bin/sh",@"/usr/bin/python",@"/usr/bin/osascript",@"/usr/bin/expect",
																  @"/usr/bin/perl",@"/usr/bin/ruby",@"/usr/bin/tclsh",@"/usr/bin/php", nil];
		for (i = 0; i < [defaultInterpreters count]; i++)
		{
			if ([shebangInterpreter caseInsensitiveCompare: [defaultInterpreters objectAtIndex: i]] == NSOrderedSame)
			{
				[self setScriptType: i];
			}
		}
		
		//set shebang interpreter into interpreter field
		[interpreterTextField setStringValue: shebangInterpreter ];
	}
	
	[self controlTextDidChange: NULL];
	
	//add to open recent menu
	if (![recentItems containsObject: filename])
	{
		if ([recentItems count] == 8)
			[recentItems removeObjectAtIndex: 0];

		[recentItems addObject: filename];
		[self constructOpenRecentMenu];
	}
}

/*****************************************
 - Toggles between advanced options mode
*****************************************/

- (IBAction)toggleAdvancedOptions:(id)sender
{
	#define		kWindowExpansionHeight  272
	NSRect winRect = [window frame];

	if ([showAdvancedArrow state] == NSOffState)
	{
		winRect.origin.y += kWindowExpansionHeight;
		winRect.size.height -= kWindowExpansionHeight;
		[showOptionsTextField setStringValue: @"Show Advanced Options"];
		[toggleAdvancedMenuItem setTitle: @"Show Advanced Options"];
		[window setFrame: winRect display:TRUE];
	}
	else if ([showAdvancedArrow state] == NSOnState)
	{
		winRect.origin.y -= kWindowExpansionHeight;
		winRect.size.height += kWindowExpansionHeight;
		[showOptionsTextField setStringValue: @"Hide advanced options"];
		[toggleAdvancedMenuItem setTitle: @"Hide Advanced Options"];
		[window setFrame: winRect display:TRUE];
	}
}

/*****************************************
 - called when [X] Is Droppable is pressed
*****************************************/
- (IBAction)isDroppableWasClicked:(id)sender
{
	[editTypesButton setHidden: ![sender state]];
	if ([sender state] == NO)
		[typesWindow performClose: self];
}

/*****************************************
 - called when (*) Output Type is changed
*****************************************/
- (IBAction)outputTypeChanged: (id)sender
{
	if ([[sender selectedCell] tag] == 2)//Text output selected
	{
		[rootPrivilegesCheckbox setIntValue: 0];
		[rootPrivilegesCheckbox setEnabled: NO];
	}
	else
		[rootPrivilegesCheckbox setEnabled: YES];
}

/*****************************************
 - called when (Clear) button is pressed -- restores fields to startup values
*****************************************/
- (IBAction)clearAllFields:(id)sender
{
	//clear all text field to start value
	[appNameTextField setStringValue: @""];
	[scriptPathTextField setStringValue: @""];
	[versionTextField setStringValue: @"1.0"];
	[signatureTextField setStringValue: @"????"];
	
	//uncheck all options
	[isDroppableCheckbox setIntValue: 0];
		[self isDroppableWasClicked: isDroppableCheckbox];
	[rootPrivilegesCheckbox setIntValue: 0];
	[remainRunningCheckbox setIntValue: 0];
	[showInDockCheckbox setIntValue: 0];
	
	//clera file list
	[self clearFileList: self];
	
	//clear suffix and types lists to default values
	[self setDefaultTypes: self];
	
	
	//set script type
	[self setScriptType: 0];
	
	//set output type
	[outputTypeRadioButtons selectCellWithTag: 0];
	[self outputTypeChanged: outputTypeRadioButtons];
	
	//update button status
	[self controlTextDidChange: NULL];
}

#pragma mark -

/*****************************************
 - Flip between icons
*****************************************/
- (IBAction)switchIcons:(id)sender
{
	[self setAppIconForType: [sender intValue]];
}

- (IBAction)nextIcon:(id)sender
{
	if ([iconToggleButton intValue] + 1 > [iconToggleButton maxValue])
		[iconToggleButton setIntValue: [iconToggleButton minValue]];
	else
		[iconToggleButton setIntValue: [iconToggleButton intValue] + 1];
	
	[self setAppIconForType: [iconToggleButton intValue]];
}

- (IBAction)previousIcon:(id)sender
{
	if ([iconToggleButton intValue] - 1 < [iconToggleButton minValue])
		[iconToggleButton setIntValue: [iconToggleButton maxValue]];
	else
		[iconToggleButton setIntValue: [iconToggleButton intValue] - 1];
	
	[self setAppIconForType: [iconToggleButton intValue]];
}

/*****************************************
 - Write an NSImage as icon to a path
*****************************************/

- (void)writeImageAsIcon: (NSImage *)img path: (NSString *)path
{
	IconFamily *iconFam = [[IconFamily alloc] initWithThumbnailsOfImage: img];
	[iconFam writeToFile: path];
	[iconFam release];
}

#pragma mark -

/*****************************************
 - Generate dictionary for Info.plist in app bundle
*****************************************/

- (NSDictionary *)getInfoPropertyListDictionary: (NSString *)appName isDroppable: (BOOL)droppable inBackground: (BOOL)background
{
	NSDictionary *infoPlist;
	NSString	 *lsui = [NSString stringWithFormat:@"%d", background];//LSUIElement value
	
	if (!droppable)
	{
		infoPlist = [NSDictionary dictionaryWithObjectsAndKeys: 
								@"English", @"CFBundleDevelopmentRegion",
								appName, @"CFBundleExecutable", 
								appName, @"CFBundleName",
								[NSString stringWithFormat: @"%@ %@ Copyright %d %@", appName, [versionTextField stringValue], [[NSCalendarDate calendarDate] yearOfCommonEra], NSFullUserName() ], @"CFBundleGetInfoString", 
								@"appIcon.icns", @"CFBundleIconFile",  
								[versionTextField stringValue], @"CFBundleVersion",  
								[self generateBundleIdentifier], @"CFBundleIdentifier",  
								lsui, @"LSUIElement",
								@"6.0", @"CFBundleInfoDictionaryVersion",
								@"APPL", @"CFBundlePackageType",
								[signatureTextField stringValue], @"CFBundleSignature",
								@"MainMenu", @"NSMainNibFile",  
								@"NSApplication", @"NSPrincipalClass",  nil];

	}
	else
	{	
			NSDictionary	*typesAndSuffixesDict = [NSDictionary dictionaryWithObjectsAndKeys:
								[suffixList getSuffixArray], @"CFBundleTypeExtensions",
								[typesList getTypesArray], @"CFBundleTypeOSTypes",//so we can open folders
								[[appFunctionRadioButtons selectedCell] title], @"CFBundleTypeRole", nil];
	
		infoPlist = [NSDictionary dictionaryWithObjectsAndKeys:
								@"English", @"CFBundleDevelopmentRegion",
								appName, @"CFBundleExecutable",
								[NSArray arrayWithObject: typesAndSuffixesDict] ,@"CFBundleDocumentTypes",
								[NSString stringWithFormat: @"%@ %@ Copyright %d %@", appName, [versionTextField stringValue], [[NSCalendarDate calendarDate] yearOfCommonEra], NSFullUserName() ], @"CFBundleGetInfoString", 
								@"appIcon.icns", @"CFBundleIconFile",  
								[versionTextField stringValue], @"CFBundleVersion", 
								[self generateBundleIdentifier], @"CFBundleIdentifier",
								lsui, @"LSUIElement",
								@"6.0", @"CFBundleInfoDictionaryVersion",
								@"APPL", @"CFBundlePackageType",
								[signatureTextField stringValue], @"CFBundleSignature",
								@"MainMenu", @"NSMainNibFile",  
								@"NSApplication", @"NSPrincipalClass", nil];
	}
	return(infoPlist);
}


/*****************************************
 - Determine script type based on a file's suffix
*****************************************/

- (int)getFileTypeFromSuffix: (NSString *)fileName
{
	if ([fileName hasSuffix: @".sh"])
		return 0;
	else if ([fileName hasSuffix: @".py"])
		return 1;
	else if ([fileName hasSuffix: @".scpt"] || [fileName hasSuffix: @".applescript"])
		return 2;
	else if ([fileName hasSuffix: @".exp"] || [fileName hasSuffix: @".expect"])
		return 3;
	else if ([fileName hasSuffix: @".pl"] || [fileName hasSuffix: @".perl"])
		return 4;
	else if ([fileName hasSuffix: @".rb"] || [fileName hasSuffix: @".rbx"])
		return 5;
	else if ([fileName hasSuffix: @".tcl"])
		return 6;
	else if ([fileName hasSuffix: @".php"])
		return 7;
	else
		return 0;
}

/*****************************************
 - //return the bundle identifier for the application
*****************************************/
- (NSString *)generateBundleIdentifier
{
	NSString	*bundleId;
	//The format is "org.username.appname"
	bundleId = [NSString stringWithFormat: @"org.%@.%@", NSUserName(), [appNameTextField stringValue]];
	bundleId = [[bundleId componentsSeparatedByString:@" "] componentsJoinedByString:@""];//no spaces
	return(bundleId);
}

#pragma mark -
/*****************************************
 - Set controls according to data in NSUserDefaults
*****************************************/
- (IBAction)showPrefs:(id)sender
{		
	[parseShebangCheckbox setState: [[NSUserDefaults standardUserDefaults] boolForKey:@"ParseShebangLineForInterpreter"]];
	[revealAppCheckbox setState: [[NSUserDefaults standardUserDefaults] boolForKey:@"RevealApplicationWhenCreated"]];
	[defaultEditorMenu setTitle: [[NSUserDefaults standardUserDefaults] stringForKey:@"DefaultEditor"]];
	
	[prefsWindow center];
	[prefsWindow makeKeyAndOrderFront: sender];
}
/*****************************************
 - Set the icons for the menu items in the Editors list
*****************************************/
- (void)setIconsForEditorMenu
{
	int i;
	NSSize	smallIconSize = { 16, 16 };

	for (i = 0; i < [defaultEditorMenu numberOfItems]; i++)
	{
		if ([[[defaultEditorMenu itemAtIndex: i] title] isEqualToString: @"Select..."] == NO && [[[defaultEditorMenu itemAtIndex: i] title] length] > 0)
		{
			NSString *appPath = [[NSWorkspace sharedWorkspace] fullPathForApplication: [[defaultEditorMenu itemAtIndex: i] title]];
			NSImage *icon = [[NSWorkspace sharedWorkspace] iconForFile: appPath];
			[icon setSize: smallIconSize];
			[[defaultEditorMenu itemAtIndex: i] setImage: icon];
		}
	}
}
/*****************************************
 - Set NSUserDefaults according to control settings
*****************************************/
- (IBAction)applyPrefs:(id)sender
{
	[[NSUserDefaults standardUserDefaults] setBool: [parseShebangCheckbox state] forKey:@"ParseShebangLineForInterpreter"];
	[[NSUserDefaults standardUserDefaults] setBool: [revealAppCheckbox state]  forKey:@"RevealApplicationWhenCreated"];
	[[NSUserDefaults standardUserDefaults] setObject: [defaultEditorMenu titleOfSelectedItem]  forKey:@"DefaultEditor"];
	
	[prefsWindow performClose: sender];
}

/*****************************************
 - For selecting any application as the external editor for script
*****************************************/
- (IBAction) selectScriptEditor:(id)sender
{
	int			result;
	NSString	*editorName;
	
	//create open panel
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	[oPanel setTitle: @"Select Editor"];
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setCanChooseDirectories: NO];
	
	//run open panel
    result = [oPanel runModalForDirectory:nil file:nil types: [NSArray arrayWithObject:@"app"]];
    if (result == NSOKButton) 
	{
		//set app name minus .app suffix
		editorName = [STUtil cutSuffix: [[oPanel filename] lastPathComponent]];
		[defaultEditorMenu setTitle: editorName];
		[self setIconsForEditorMenu];
	}
	else
		[defaultEditorMenu setTitle: [[NSUserDefaults standardUserDefaults] stringForKey:@"DefaultEditor"]];
}

#pragma mark -

/*****************************************
 - Dragging and dropping for Platypus window
*****************************************/

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
	NSPasteboard	*pboard = [sender draggingPasteboard];
	NSString		*filename;
	BOOL			isDir = FALSE;
	int				i = 0;

    if ( [[pboard types] containsObject:NSFilenamesPboardType] ) 
	{
        NSArray *files = [pboard propertyListForType:NSFilenamesPboardType];

		//files were dropped in the file list table
		if (NSMouseInRect([sender draggingLocation], NSMakeRect(22, 34, 568, 145), NO) && [showAdvancedArrow state] == NSOnState)
		{
			for (i = 0; i < [files count]; i++)
			{
				//we don't add if it already exists in the array
				if ([fileList hasFile: [files objectAtIndex:i]] == NO)
					[fileList addFile: [files objectAtIndex:i]];
			}
			
			//reload data in databrowser
			[fileListDataBrowser setDataSource: fileList];
			[fileListDataBrowser reloadData];
			//update button status
			[self tableViewSelectionDidChange: NULL];
			//update size field
			[self updateFileSizeField];
			return YES;
		}
		else //files were dropped on the window
		{
			filename = [files objectAtIndex: 0];//we only load the first dragged item
			if ([[NSFileManager defaultManager] fileExistsAtPath: filename isDirectory:&isDir] && !isDir)
			{
				[self loadScript: filename];
				return YES;
			}
		}
    }

    
	return NO;
}

- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender 
{

    NSPasteboard *pboard;
    NSDragOperation sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];
    pboard = [sender draggingPasteboard];


    if ( [[pboard types] containsObject:NSFilenamesPboardType] ) 
	{
        if (sourceDragMask & NSDragOperationLink) 
            return NSDragOperationLink;
		else if (sourceDragMask & NSDragOperationCopy)
            return NSDragOperationCopy;
    }

    return NSDragOperationNone;
}

#pragma mark -

/*****************************************
 - Delegate managing selection in the Bundled Files list
*****************************************/

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	int i;
	int selected = 0;
	NSIndexSet *selectedItems = [fileListDataBrowser selectedRowIndexes];
	
	for (i = 0; i < [fileList numFiles]; i++)
	{
		if ([selectedItems containsIndex: i])
		{
			selected++;
		}
	}
	
	//update button status
	if (selected == 0)
	{
		[removeFileButton setEnabled: NO];
		[revealFileButton setEnabled: NO];
	}
	else
	{
		[removeFileButton setEnabled: YES];
		[revealFileButton setEnabled: YES];
	}
	
	if ([fileList numFiles] == 0)
		[clearFileListButton setEnabled: NO];
	else
		[clearFileListButton setEnabled: YES];	
}

/*****************************************
 - Updates text field listing total size of bundled files
*****************************************/
- (void)updateFileSizeField
{
	int			i;
	UInt64		totalSize = 0;
	NSString	*totalSizeString;

	//if there are no items, we just list it as 0 items
	if ([fileList numFiles] <= 0)
	{
		[bundleSizeTextField setStringValue: [NSString stringWithFormat: @"%d items", [fileList numFiles]]];
		return;
	}
	
	//otherwise, loop through all files, calculate size
	for (i = 0; i < [fileList numFiles]; i++)
	{		
		totalSize += [STUtil fileOrFolderSize: [fileList getFileAtIndex: i]];
	}
	
	totalSizeString = [STUtil sizeAsHumanReadable: totalSize];
	if ([fileList numFiles] > 1)
		[bundleSizeTextField setStringValue: [NSString stringWithFormat: @"%d items  ( %@ )", [fileList numFiles], totalSizeString]];
	else
		[bundleSizeTextField setStringValue: [NSString stringWithFormat: @"%d item  ( %@ )", [fileList numFiles], totalSizeString]];
}


/*****************************************
 - Delegate for when text changes in any of the Platypus text fields
*****************************************/
- (void)controlTextDidChange:(NSNotification *)aNotification
{
	BOOL	isDir, exists = NO, validName = NO;
	
	//app name or script path was changed
	if ([aNotification object] == NULL || [aNotification object] == appNameTextField || [aNotification object] == scriptPathTextField)
	{
		if ([[appNameTextField stringValue] length] > 0)
			validName = YES;
		if ([[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] isDirectory:&isDir] && !isDir)
			exists = YES;
		
		//edit and reveal buttons
		if (exists)
		{
			[editScriptButton setEnabled: YES];
			[revealScriptButton setEnabled: YES];
		}
		else
		{
			[editScriptButton setEnabled: NO];
			[revealScriptButton setEnabled: NO];
		}
		
		//enable/disable create app button
		if (validName && exists)
			[createAppButton setEnabled: YES];
		else
			[createAppButton setEnabled: NO];
		
		//update identifier
		if (validName)
			[bundleIdentifierTextField setStringValue: [NSString stringWithFormat: @"Bundle identifier will be %@", [self generateBundleIdentifier]]];
		else
			[bundleIdentifierTextField setStringValue: @""];
	}
	
	//bundle signature changed
	if ([aNotification object] == signatureTextField || [aNotification object] == NULL)
	{
		NSRange	 range = { 0, 4 };
		NSString *sig = [signatureTextField stringValue];
		
		if ([sig length] > 4)
		{
			[signatureTextField setStringValue: [sig substringWithRange: range]];
		}
	}
}

/*****************************************
 - Delegate for enabling and disabling menu items
*****************************************/
- (BOOL)validateMenuItem:(NSMenuItem*)anItem 
{
	BOOL isDir;

	//edit script
	if ([[anItem title] isEqualToString:@"Edit Script"] && (![[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] isDirectory:&isDir] || isDir))
	{
		return NO;
	}
	//reveal script
	if ([[anItem title] isEqualToString:@"Reveal Script"] && (![[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] isDirectory:&isDir] || isDir))
	{
		return NO;
	}
	//run script
	if ([[anItem title] isEqualToString:@"Run Script in Terminal"] && (![[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] isDirectory:&isDir] || isDir))
	{
		return NO;
	}
	//create app menu
	if ([[anItem title] isEqualToString:@"Create App"] && (![[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] isDirectory:&isDir] || isDir))
	{
		return NO;
	}

	//File list menu items
	if ([[anItem title] isEqualToString:@"Add File To Bundle"] && [showAdvancedArrow state] == NSOffState)
		return NO;
		
    if ([[anItem title] isEqualToString:@"Remove File"] && ([fileList numFiles] == 0 || [showAdvancedArrow state] == NSOffState))
		return NO;

    if ([[anItem title] isEqualToString:@"Reveal File In File List"] && ([fileList numFiles] == 0 || [showAdvancedArrow state] == NSOffState))
		return NO;
	
	if ([[anItem title] isEqualToString:@"Clear File List"] && ([fileList numFiles] == 0 || [showAdvancedArrow state] == NSOffState))
		return NO;

	//Profiles menu
	if ([[anItem title] isEqualToString:@"Save Profile"] && ([[appNameTextField stringValue] length] == 0 ||[[scriptPathTextField stringValue] length] == 0))
		return NO;

	if ([[anItem title] isEqualToString:@"Clear All Profiles"] && ![[[NSUserDefaults standardUserDefaults] objectForKey:@"Profiles"] count])
		return NO;

    return YES;

}

/*****************************************
 - Load the script of item selected in Open Recent Menu
*****************************************/
-(void) openRecentMenuItemSelected: (id)sender
{
	BOOL	isDir = NO;

	if ([[NSFileManager defaultManager] fileExistsAtPath: [sender title] isDirectory: &isDir] && isDir == NO)
		[self loadScript: [sender title]];
	else
		[STUtil alert:@"Invalid item" subText: @"The file you selected no longer exists at the specified path."];
}
/*****************************************
 - Generate the Open Recent Menu
*****************************************/
-(void) constructOpenRecentMenu
{
	int i;

	//clear out all menu itesm
	while ([openRecentMenu numberOfItems])
		[openRecentMenu removeItemAtIndex: 0];

	if ([recentItems count] > 0)
	{
		//populate with contents of array
		for (i = [recentItems count]-1; i >= 0 ; i--)
		{
			[openRecentMenu addItemWithTitle: [recentItems objectAtIndex: i] action: @selector(openRecentMenuItemSelected:) keyEquivalent:@""];
		}
		
		//add seperator and clear menu
		[openRecentMenu addItem: [NSMenuItem separatorItem]];
		[openRecentMenu addItemWithTitle: @"Clear" action: @selector(clearRecentItems) keyEquivalent:@""];
		
	}
	else
	{
		[openRecentMenu addItemWithTitle: @"Empty" action: NULL keyEquivalent:@""];
		[[openRecentMenu itemAtIndex: 0] setEnabled: NO];
	}
}
/*****************************************
 - Clear the Recent Items menu
*****************************************/
-(void) clearRecentItems
{
	[recentItems removeAllObjects];
	[self constructOpenRecentMenu];
}

#pragma mark -


/*****************************************
 - called when + button is pressed
*****************************************/

- (IBAction)addFileToFileList:(id)sender
{
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	[oPanel setPrompt:@"Add"];
	[oPanel setCanChooseDirectories:YES];
    [oPanel setAllowsMultipleSelection:YES];
	
	[window setTitle: @"Platypus - Select files or folders to add"];
	
	//run open panel
    [oPanel beginSheetForDirectory:nil file:nil types:nil modalForWindow: window modalDelegate: self didEndSelector: @selector(addFilesPanelDidEnd:returnCode:contextInfo:) contextInfo: nil];
}

- (void)addFilesPanelDidEnd:(NSOpenPanel *)oPanel returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if (returnCode == NSOKButton)
	{
		//add all the files to the file list
        NSArray *filesToOpen = [oPanel filenames];
        int i, count = [filesToOpen count];
        for (i=0; i<count; i++) 
		{
			//we don't add if it already exists in the array
			if ([fileList hasFile: [filesToOpen objectAtIndex:i]] == NO)
				[fileList addFile: [filesToOpen objectAtIndex:i]];
        }
		//reload data
		[fileListDataBrowser setDataSource: fileList];
		[fileListDataBrowser reloadData];
		//update button status
		[self tableViewSelectionDidChange: NULL];
		[self updateFileSizeField];
	}
	
	[window setTitle: @"Platypus"];
}

/*****************************************
 - called when C button is pressed
*****************************************/

- (IBAction)clearFileList:(id)sender
{
	[fileList clearList];
	[fileListDataBrowser reloadData];
	//update button status
	[self tableViewSelectionDidChange: NULL];
	[self updateFileSizeField];
}


/*****************************************
 - called when R button is pressed
*****************************************/
- (IBAction)revealFileInFileList:(id)sender
{	
	int i;
	NSIndexSet *selectedItems = [fileListDataBrowser selectedRowIndexes];
	
	for (i = 0; i < [fileList numFiles]; i++)
	{
		if ([selectedItems containsIndex: i])
		{
			[fileList revealInFinder: i];
		}
	}
}

/*****************************************
 - called when - button is pressed
*****************************************/

- (IBAction)removeFileFromFileList:(id)sender
{
	int i, didRemove = FALSE;
	NSIndexSet *selectedItems = [fileListDataBrowser selectedRowIndexes];
	
	for (i = [fileList numFiles]; i >= 0; i--)
	{
		if ([selectedItems containsIndex: i])
		{
			[fileList removeFile: i];
			didRemove = TRUE;
		}
	}
	
	if (didRemove == TRUE)
		[fileListDataBrowser reloadData];
	
	//update size field
	[self updateFileSizeField];
}


#pragma mark -

/*****************************************
 - Save a profile with values from fields
*****************************************/
- (IBAction) saveProfile:(id)sender;
{
	if (![self verifyFieldContents])//are there invalid values in the fields?
		return;
	
	NSDictionary *profileDict = [NSDictionary dictionaryWithObjectsAndKeys:
					[appNameTextField stringValue], @"AppName",
					[scriptPathTextField stringValue], @"ScriptPath",
					[NSNumber numberWithInt: [[outputTypeRadioButtons selectedCell] tag]], @"OutputType",
					[NSNumber numberWithInt: [[scriptTypeRadioButtons selectedCell] tag]], @"ScriptType",
					[interpreterTextField stringValue], @"Interpreter",
					[versionTextField stringValue], @"Version",
					[signatureTextField stringValue], @"Signature",
					[NSNumber numberWithBool: [rootPrivilegesCheckbox intValue]], @"RequiresAdminPrivileges",
					[NSNumber numberWithBool: [isDroppableCheckbox intValue]], @"IsDroppable",
					[NSNumber numberWithBool: [showInDockCheckbox intValue]], @"RunsInBackground",
					[NSNumber numberWithBool: [remainRunningCheckbox intValue]], @"RemainRunningAfterCompletion",
					[fileList getFilesArray], @"BundledFiles",
					[typesList getTypesArray], @"Types",
					[suffixList getSuffixArray], @"Suffixes",
					[NSNumber numberWithInt: [[appFunctionRadioButtons selectedCell] tag]], @"AppFunction", nil];
	
	NSMutableArray *profiles = [NSMutableArray arrayWithCapacity: 255];
	[profiles addObjectsFromArray: [[NSUserDefaults standardUserDefaults] objectForKey:@"Profiles"]];
	[profiles addObject: profileDict];
	[[NSUserDefaults standardUserDefaults] setObject: profiles  forKey:@"Profiles"];
	//regenerate Profiles menu
	[self constructProfilesMenu];
}


/*****************************************
 - Fill Platypus fields in with data from profile when it's selected
*****************************************/
-(void) profileMenuItemSelected: (id)sender
{
	int				itemNum = [profilesMenu indexOfItem: sender] - 3;
	NSDictionary	*profile = [[[NSUserDefaults standardUserDefaults] objectForKey:@"Profiles"] objectAtIndex: itemNum];
	
	//text fields
	[appNameTextField setStringValue: [profile objectForKey: @"AppName"]];
	[scriptPathTextField setStringValue: [profile objectForKey: @"ScriptPath"]];
	
	[versionTextField setStringValue: [profile objectForKey: @"Version"]];
	[signatureTextField setStringValue: [profile objectForKey: @"Signature"]];
	
	//radio buttons
		//set output type
		[outputTypeRadioButtons selectCellWithTag: [[profile objectForKey: @"OutputType"] intValue]];
		[self outputTypeChanged: outputTypeRadioButtons];
		[self setScriptType: [[profile objectForKey: @"ScriptType"] intValue]];//script type
		[interpreterTextField setStringValue: [profile objectForKey: @"Interpreter"]];
		
	//checkboxes
	[rootPrivilegesCheckbox setIntValue: [[profile objectForKey: @"RequiresAdminPrivileges"] boolValue]];
	[isDroppableCheckbox setIntValue: [[profile objectForKey: @"IsDroppable"] boolValue]];
		[self isDroppableWasClicked: isDroppableCheckbox];
	[showInDockCheckbox setIntValue: [[profile objectForKey: @"RunsInBackground"] boolValue]];
	[remainRunningCheckbox setIntValue: [[profile objectForKey: @"RemainRunningAfterCompletion"] boolValue]];
	
	//file list
		[fileList clearList];
		[fileList addFiles: [profile objectForKey: @"BundledFiles"]];
		//reload data
		[fileListDataBrowser setDataSource: fileList];
		[fileListDataBrowser reloadData];
		//update button status
		[self tableViewSelectionDidChange: NULL];
		[self updateFileSizeField];
	
	//suffix list
		[suffixList clearList];
		[suffixList addSuffixes: [profile objectForKey: @"Suffixes"]];
		[suffixListDataBrowser setDataSource: suffixList];
		[suffixListDataBrowser reloadData];
	
		if ([suffixList hasAllSuffixes])
			[numSuffixesTextField setStringValue: @"All suffixes"];
		else
			[numSuffixesTextField setStringValue: [NSString stringWithFormat:@"%d suffixes", [suffixList numSuffixes]]];
	//types list
		[typesList clearList];
		[typesList addTypes: [profile objectForKey: @"Types"]];
		[typesListDataBrowser setDataSource: typesList];
		[typesListDataBrowser reloadData];
		
		if ([typesList hasAllTypes])
			[numTypesTextField setStringValue: @"All file types"];
		else
			[numTypesTextField setStringValue: [NSString stringWithFormat:@"%d file types", [typesList numTypes]]];
		if ([typesList hasFolderType])
			[numTypesTextField setStringValue: [[numTypesTextField stringValue] stringByAppendingString: @" and folders"]];
	//set app function
		[appFunctionRadioButtons selectCellWithTag: [[profile objectForKey: @"AppFunction"] intValue]];
}

- (IBAction) clearAllProfiles:(id)sender;
{
	//clear the array with the profiles data
	[[NSUserDefaults standardUserDefaults] setObject: [NSArray array]  forKey:@"Profiles"];
	//regenerate the menu
	[self constructProfilesMenu];
}

/*****************************************
 - Generate the Profiles menu according to the save profiles
*****************************************/
- (void) constructProfilesMenu
{
	int i;
	NSArray *profiles = [[NSUserDefaults standardUserDefaults] objectForKey:@"Profiles"];

	//clear out all menu itesm
	while ([profilesMenu numberOfItems] > 3)
		[profilesMenu removeItemAtIndex: 3];

	if ([profiles count] > 0)
	{
		//populate with contents of array
		for (i = 0; i < [profiles count]; i++)
		{
			NSString	*menuItemTitle = [NSString stringWithFormat: @"%@ (%@)", [[profiles objectAtIndex: i] objectForKey: @"AppName"], [[profiles objectAtIndex: i] objectForKey: @"ScriptPath"]];
			[profilesMenu addItemWithTitle: menuItemTitle action: @selector(profileMenuItemSelected:) keyEquivalent:@""];
		}
	}
	else
	{
		[profilesMenu addItemWithTitle: @"Empty" action: NULL keyEquivalent:@""];
	}
}

#pragma mark -


/*****************************************
 - called when [+] button is pressed in Types List
*****************************************/

- (IBAction) addType:(id)sender;
{
	//make sure the type is 4 characters long
	if ([[typeCodeTextField stringValue] length] != 4)
	{
				[STUtil sheetAlert:@"Invalid File Type" subText: @"A File Type must consist of exactly 4 ASCII characters." forWindow: typesWindow];
		return;
	}

	if (![typesList hasType: [typeCodeTextField stringValue]] && ([[typeCodeTextField stringValue] length] > 0))
	{
		[typesList addType: [typeCodeTextField stringValue]];
		[typeCodeTextField setStringValue: @""];
	}
	//update
	[typesListDataBrowser setDataSource: typesList];
	[typesListDataBrowser reloadData];
	
	if ([typesList hasAllTypes])
		[numTypesTextField setStringValue: @"All file types"];
	else
		[numTypesTextField setStringValue: [NSString stringWithFormat:@"%d file types", [typesList numTypes]]];
	if ([typesList hasFolderType])
		[numTypesTextField setStringValue: [[numTypesTextField stringValue] stringByAppendingString: @" and folders"]];
}

/*****************************************
 - called when [-] button is pressed in Types List
*****************************************/

- (IBAction) removeType:(id)sender;
{
	int i;
	NSIndexSet *selectedItems = [typesListDataBrowser selectedRowIndexes];
	
	for (i = [typesList numTypes]; i >= 0; i--)
	{
		if ([selectedItems containsIndex: i])
		{
			[typesList removeType: i];
			[typesListDataBrowser reloadData];
			break;
		}
	}
	
	if ([typesList hasAllTypes])
		[numTypesTextField setStringValue: @"All file types"];
	else
		[numTypesTextField setStringValue: [NSString stringWithFormat:@"%d file types", [typesList numTypes]]];
	if ([typesList hasFolderType])
		[numTypesTextField setStringValue: [[numTypesTextField stringValue] stringByAppendingString: @" and folders"]];
}

/*****************************************
 - called when [C] button is pressed in Types List
*****************************************/
- (IBAction) clearTypesList:(id)sender
{
	[typesList clearList];
	[typesListDataBrowser reloadData];
	[numTypesTextField setStringValue: [NSString stringWithFormat:@"%d file types", [typesList numTypes]]];
}

/*****************************************
 - called when [+] button is pressed in Types List
*****************************************/

- (IBAction) addSuffix:(id)sender;
{
	if (![suffixList hasSuffix: [suffixTextField stringValue]] && ([[suffixTextField stringValue] length] > 0))
	{
		[suffixList addSuffix: [suffixTextField stringValue]];
		[suffixTextField setStringValue: @""];
	}
	//update
	[suffixListDataBrowser setDataSource: suffixList];
	[suffixListDataBrowser reloadData];
	
	if ([suffixList hasAllSuffixes])
		[numSuffixesTextField setStringValue: @"All suffixes"];
	else
		[numSuffixesTextField setStringValue: [NSString stringWithFormat:@"%d suffixes", [suffixList numSuffixes]]];
}

/*****************************************
 - called when [-] button is pressed in Types List
*****************************************/

- (IBAction) removeSuffix:(id)sender;
{
	int i;
	NSIndexSet *selectedItems = [suffixListDataBrowser selectedRowIndexes];
	
	for (i = [suffixList numSuffixes]; i >= 0; i--)
	{
		if ([selectedItems containsIndex: i])
		{
			[suffixList removeSuffix: i];
			[suffixListDataBrowser reloadData];
			break;
		}
	}
	
	if ([suffixList hasAllSuffixes])
		[numSuffixesTextField setStringValue: @"All suffixes"];
	else
		[numTypesTextField setStringValue: [NSString stringWithFormat:@"%d suffixes", [suffixList numSuffixes]]];
}

/*****************************************
 - called when [C] button is pressed in Types List
*****************************************/
- (IBAction) clearSuffixList:(id)sender
{
	[suffixList clearList];
	[suffixListDataBrowser reloadData];
	[numSuffixesTextField setStringValue: [NSString stringWithFormat:@"%d suffixes", [suffixList numSuffixes]]];
}



/*****************************************
 - called when "Default" button is pressed in Types List
*****************************************/
- (IBAction) setDefaultTypes:(id)sender
{
	//default File Types
	[typesList clearList];
	[typesList addType: @"****"];
	[typesList addType: @"fold"];
	[typesListDataBrowser reloadData];
	
		if ([typesList hasAllTypes])
		[numTypesTextField setStringValue: @"All file types"];
	else
		[numTypesTextField setStringValue: [NSString stringWithFormat:@"%d file types", [typesList numTypes]]];
	if ([typesList hasFolderType])
		[numTypesTextField setStringValue: [[numTypesTextField stringValue] stringByAppendingString: @" and folders"]];
		
	//default suffixes
	[suffixList clearList];
	[suffixList addSuffix: @"*"];
	[suffixListDataBrowser reloadData];
	
	if ([suffixList hasAllSuffixes])
		[numSuffixesTextField setStringValue: @"All suffixes"];
	else
		[numSuffixesTextField setStringValue: [NSString stringWithFormat:@"%d suffixes", [suffixList numSuffixes]]];
	
	//set app function to default
	[appFunctionRadioButtons selectCellWithTag: 0];
}

#pragma mark -

/*****************************************
 - Open Platypus Help HTML file within app bundle
*****************************************/
- (IBAction) showHelp:(id)sender
{
	NSString	*thePath = [[NSBundle mainBundle] pathForResource:@"index.html" ofType:nil];
	[NSTask launchedTaskWithLaunchPath: @"/usr/bin/open" arguments: [NSArray arrayWithObject: thePath]];
}

@end
