/*
    ScriptExec - binary bundled into Platypus-created applications
    Copyright (C) 2003 Sveinbjorn Thordarson <sveinbt@hi.is>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "ScriptExecController.h"
#include <Security/Authorization.h>
#include <sys/wait.h>
#include <stdio.h>
#include <fcntl.h>

@implementation ScriptExecController

- (id)init
{
	if (self = [super init]) 
	{
		//create arrays
		arguments = [[NSMutableArray alloc] initWithCapacity: kMaxFileArguments+2];
		fileArgs = [[NSMutableArray alloc] initWithCapacity: kMaxFileArguments];
		isTaskDone = NO;
		isTaskStarted = NO;
    }
    return self;
}

-(void)dealloc
{
	[arguments release];
	[fileArgs release];
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	//load settings from app bundle
	[self loadSettings];
	
	//put application name into the relevant menu items
	[quitMenuItem setTitle: [NSString stringWithFormat: @"Quit %@", appName]];
	[aboutMenuItem setTitle: [NSString stringWithFormat: @"About %@", appName]];
	[hideMenuItem setTitle: [NSString stringWithFormat: @"Hide %@", appName]];
	
	//create progress window if so specified
	if (outputType == kProgressBarOutput)
	{
		//prepare progress bar, then center and display window
		[progressBar setUsesThreadedAnimation: YES];
		[progressBar startAnimation: self];
		[progressWindow setTitle: appName];
		[progressWindow center];
		[progressWindow makeKeyAndOrderFront: NULL];
	}
	else if (outputType == kTextOutput)
	{   
		//prepare text window controls, then center and display the window
		[textOutputTextField setFont:[NSFont userFixedPitchFontOfSize: 10.0]];
		[textOutputProgressIndicator setUsesThreadedAnimation: YES];
		[textOutputProgressIndicator startAnimation: self];
		[textOutputWindow setTitle: appName];
		[textOutputWindow center];
		[textOutputWindow makeKeyAndOrderFront: NULL];
	}

	//create argument list
	[arguments addObject: scriptPath];
	[arguments addObject: [[NSBundle mainBundle] bundlePath]]; //$1 is path of application bundle
	if ([fileArgs count] > 0)//if there are any dropped files, we add them as arguments after $1
		[arguments addObjectsFromArray: fileArgs];

	//start new thread for executing script
	if (execStyle == kPrivilegedExecution) //Authentication mode
	{
		//[NSThread detachNewThreadSelector: @selector(executeScriptWithPrivileges) toTarget: self withObject: NULL];
		[self executeScriptWithPrivileges];
	}
	else //plain old regular
	{
		//[NSThread detachNewThreadSelector: @selector(executeScript) toTarget: self withObject: NULL];
		[self executeScript];
	}
}

#pragma mark -

//launch regular user task with NSTask
- (void)executeScript
{	
	isTaskStarted = YES;

	//initalize task
	task = [[NSTask alloc] init];
	
	if (outputType == kTextOutput)// we monitor output if TextWindow output is on
	{
		outputPipe = [NSPipe pipe];
		[task setStandardOutput: outputPipe];
		[task setStandardError: outputPipe];
		readHandle = [outputPipe fileHandleForReading];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(getData:) name:NSFileHandleReadCompletionNotification object:readHandle];
		[readHandle readInBackgroundAndNotify];
	}
	
	//apply settings for task
	[task setLaunchPath: interpreter];
	[task setArguments: arguments];
	
	//set it off
	[task launch];

	[NSTimer scheduledTimerWithTimeInterval: 0.25 target: self selector:@selector(checkTaskStatus) userInfo: nil repeats: YES];
}

// check if task is running
- (void)checkTaskStatus
{
	if (![task isRunning])
		[self taskFinished];
}

#pragma mark -


//launch task with privileges using Authentication Manager
- (void)executeScriptWithPrivileges
{
	OSErr					err = noErr;
    short					i;
    char					*args[kMaxFileArguments+2];
	char					interpreterStr[kMaxPathLength];
	FILE					*outputFile;
	
	isTaskStarted = YES;
	
	//interpreter
	[interpreter getCString: (char *)&interpreterStr maxLength: kMaxPathLength];
	
	//create arguments array
	for (i = 0; i < [arguments count]; i++)
	{
		args[i] = malloc(kMaxPathLength);
		[[arguments objectAtIndex:i] getCString: (char *)args[i] maxLength: kMaxPathLength];
		if (i == [arguments count]-1)
		{
			args[i+1] = NULL;
		}
	}
    
    // Use Apple's Authentication Manager APIs to get an Authorization Reference
    err = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, &authorizationRef);
    if (err != errAuthorizationSuccess)
	{
		NSLog(@"Authorization for script execution failed - Error %d", err);
        [[NSApplication sharedApplication] terminate: self];
	}
	//use Authorization Reference to execute script with privileges
    if (!(err = AuthorizationExecuteWithPrivileges(authorizationRef,(char *)&interpreterStr, kAuthorizationFlagDefaults, args, &outputFile)) != noErr)
	{
		//get NSFileHandle for the task output
		readHandle = [[NSFileHandle alloc] initWithFileDescriptor: fileno(outputFile)];
		childPid = fcntl(fileno(outputFile), F_GETOWN, 0);//get pid
		// read the filehandle in the background and regularly print output into text window
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(getData:) name:NSFileHandleReadCompletionNotification object:readHandle];
		[readHandle readInBackgroundAndNotify];
		[NSTimer scheduledTimerWithTimeInterval: 0.25 target: self selector:@selector(checkPrivilegedTaskStatus) userInfo: nil repeats: YES];
	}
	else
	{
		NSLog(@"Error %d occurred when attempting to run AuthorizationExecuteWithPrivileges. Terminating...", err);
		[[NSApplication sharedApplication] terminate: self];
	}
}

// check if privileged task is running
- (void)checkPrivilegedTaskStatus
{
    int ret,pid;
    pid = waitpid(childPid, &ret, WNOHANG);
    if (pid != 0)
        [self taskFinished];
}

#pragma mark -

// OK, called when task is finished.  Some cleaning up to do, controls need to be adjusted, etc.
- (void)taskFinished
{
		if (!remainRunning)
		{
			// we quit if the app isn't explicity set to continue running
			[[NSApplication sharedApplication] terminate: self];
		}
		else if (outputType == kTextOutput)
		{
			//update controls for text output window
			[textOutputCancelButton setTitle: @"Quit"];
			[textOutputCancelButton setKeyEquivalent:@"\r"];
			[textOutputProgressIndicator stopAnimation: self];
		}
		else if (outputType == kProgressBarOutput)
		{
			//update controls for progress bar output
			[messageTextField setStringValue: @"Task completed"];
			[progressBar stopAnimation: self];
			[cancelButton setTitle: @"Quit"];
			[cancelButton setKeyEquivalent:@"\r"];
		}
		isTaskDone = YES;
}


// read from the file handle and append it to the text window
- (void) getData: (NSNotification *)aNotification
{
	NSData *data;
	id notificationObject;

	//get the data
	notificationObject = [aNotification object];
	data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];
	
	//make sure there's actual data
	if ([data length]) 
	{
		//append the output
		outputStr = [[[NSString alloc] initWithData: data encoding: NSASCIIStringEncoding] autorelease];
		[textOutputTextField setString: [[textOutputTextField string] stringByAppendingString: outputStr] ];
		//scroll to it
		[textOutputTextField scrollRangeToVisible:NSMakeRange([[textOutputTextField string] length], 0)];
		// we schedule the file handle to go read more data in the background again.
		[notificationObject readInBackgroundAndNotify];
	}
}



#pragma mark -

// respond to AEOpenDoc -- so MUCH more convenient than working with Apple Event Descriptors!
- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
{
	if (!isTaskStarted)
	{
		[fileArgs addObject: filename];
		return TRUE;
	}
	else
		return FALSE;
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
	char	killProgStr[] = "/bin/kill";
	char	childPidStr[24];
	char	*args[2] = { NULL, NULL };
	
	//we need to terminate task under authorization if it was run as authorized and is still running
	if (execStyle == kPrivilegedExecution && !isTaskDone)
	{
		//release file handle
		if (readHandle != NULL)
			[readHandle release];
		
		//create argument array for 'kill', which just contains the PID
		sprintf((char *)&childPidStr, "%d", childPid);
		args[0] = (char *)&childPidStr;
		
		//run the kill command on the process
		if (AuthorizationExecuteWithPrivileges(authorizationRef,(char *)&killProgStr, kAuthorizationFlagDefaults, args, NULL))
			NSLog(@"Error terminating Authorized task PID %d", childPid);
	}
	else // just regular old user process
	{
		//terminate task
		if (task != NULL) //&& [task isRunning])
		{
			if ([task isRunning])
				[task terminate];
			[task release];
		}
	}
	return(YES);
}

#pragma mark -

//load configuration files from application bundle
- (void)loadSettings
{
	NSBundle		*appBundle = [NSBundle mainBundle];
	NSFileManager   *fmgr = [NSFileManager defaultManager];
	NSDictionary	*appSettingsPlist;
	
	//make sure all the config files are present -- if not, we quit
	if (![fmgr fileExistsAtPath: [appBundle pathForResource:@"AppSettings.plist" ofType:nil]] || ![fmgr fileExistsAtPath: [appBundle pathForResource:@"script" ofType:nil]])
		[self fatalAlert: @"Corrupted app bundle" subText: @"Vital configuration files were missing from the application bundle."];
	
	//get app name
	appName = [[appBundle executablePath] lastPathComponent];
	
	//get dictionary with app settings
	appSettingsPlist = [NSDictionary dictionaryWithContentsOfFile: [appBundle pathForResource:@"AppSettings.plist" ofType:nil]];
	
	//get interpreter
	interpreter = [appSettingsPlist objectForKey:@"ScriptInterpreter"];
		
	//get path to script
	scriptPath = [appBundle pathForResource:@"script" ofType:nil];
	
	//determine output type
	NSString *outputTypeStr = [appSettingsPlist objectForKey:@"OutputType"];
	if ([outputTypeStr isEqualToString: @"Progress Bar"])
		outputType = kProgressBarOutput;
	else if ([outputTypeStr isEqualToString: @"Text Window"])
		outputType = kTextOutput;
	else
		outputType = kNoOutput;
		
	//determine execution style
	execStyle = [[appSettingsPlist objectForKey:@"RequiresAdminPrivileges"] boolValue];

	//remain running?
	remainRunning = [[appSettingsPlist objectForKey:@"RemainRunningAfterCompletion"] boolValue];
}



// Respond to Cancel by exiting application
- (IBAction)cancel:(id)sender
{
	[[NSApplication sharedApplication] terminate: self];
}

#pragma mark -

- (void)fatalAlert: (NSString *)message subText: (NSString *)subtext
{
	NSAlert *alert = [[NSAlert alloc] init];
	[alert addButtonWithTitle:@"OK"];
	[alert setMessageText: message];
	[alert setInformativeText: subtext];
	[alert setAlertStyle: NSCriticalAlertStyle];
	
	if ([alert runModal] == NSAlertFirstButtonReturn) 
	{
		[alert release];
		ExitToShell();
	} 
}


@end
