
/*
    Platypus - create MacOS X application bundles that execute scripts
    Copyright (C) 2003 Sveinbjorn Thordarson <sveinbt@hi.is>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    util.c - Utility functions

*/


#include <Carbon/Carbon.h>
#include "platypus.h"
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>

////////////////////////////////////////
// Routine to register an icns file.
///////////////////////////////////////
IconRef RegisterIcon( FourCharCode inCreator, FourCharCode inType, const char* inName )
{
	IconRef result = 0;
	OSErr err = fnfErr;
	
	if ( inName != NULL && *inName != 0 )
	{
		CFBundleRef appBundle = CFBundleGetMainBundle();
		if ( appBundle )
		{
			CFStringRef fileName = CFStringCreateWithCString( NULL, inName, kCFStringEncodingASCII );
			if (fileName != NULL)
			{
				CFURLRef iconFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
				if (iconFileURL != NULL)
				{
					FSRef		fileRef;
					
					if ( CFURLGetFSRef( iconFileURL, &fileRef ) )
					{
						FSSpec		iconFileSpec;
						
						err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &iconFileSpec, NULL );
						if ( err == noErr )
							err = RegisterIconRefFromIconFile( inCreator, inType, &iconFileSpec, &result );
					}
					CFRelease( iconFileURL );
				}
				CFRelease( fileName );
			}
		}
	}
	
	return result;
}


////////////////////////////////////////
//Standard error alert
////////////////////////////////////////
void RedAlert (Str255 errorString, Str255 expStr)
{
    StandardAlert (kAlertStopAlert, errorString,  expStr, NULL, NULL);
}


////////////////////////////////////////
//Standard error alert, then exit application
////////////////////////////////////////
void RedFatalAlert (Str255 errorString, Str255 expStr)
{
    StandardAlert (kAlertStopAlert, errorString,  expStr, NULL, NULL);
    QuitApp();
}

////////////////////////////////////////
//Standard warning alert
////////////////////////////////////////
void YellowAlert (Str255 errorString, Str255 expStr)
{
    StandardAlert (kAlertCautionAlert, errorString,  expStr, NULL, NULL);
}


////////////////////////////////////////
// Open URL with default browser using
// MacOS X's 'open' CLI command
///////////////////////////////////////
void OpenURL (char *url)
{
   	char	command[2048] = "open '";

	//add defined URL
        strcat((char *)&command, url);
        strcat((char *)&command, "'");
	//pass command to the system
	system((char *)&command);
}

////////////////////////////////////////
// Send system a command to open mail
// client and point to specified address
///////////////////////////////////////
void SendMail (char *email)
{
	char	command[512] = "open \"mailto:";

	//add defined URL
        strcat((char *)&command, email);
        strcat((char *)&command, "\"");
	//pass command to the system
	system((char *)&command);

}

////////////////////////////////////////
// Check if file in designated path is folder
///////////////////////////////////////
short UnixIsFolder (char *path)
{
    struct stat filestat;
    short err;
    
    err = stat(path, &filestat);
    if (err == -1)
        return err;

    if(S_ISREG(filestat.st_mode) != 1)
        return true;
    else 
        return false;
}



////////////////////////////////////////
// Cut the last segment of a path name 
// away from the string
///////////////////////////////////////
void CutFileFromPath (char *path)
{
    short i;
    short len = strlen(path);
    
    for (i = len; i > 0; i--)
    {
        if (path[i] == '/')
        {
            path[i] = '\0';
            break;
        }
    }
}

////////////////////////////////////////
// Generate path string from FSSpec record
///////////////////////////////////////
OSStatus FSMakePath(FSSpec file, UInt8 *path, UInt32 maxPathSize)
{
        OSStatus result;
        FSRef  ref;
 
        FSpMakeFSRef(&file, &ref);
 
 	/* and then convert the FSRef to a path */
	result = FSRefMakePath(&ref, path, maxPathSize);
        return ( result );
}


////////////////////////////////////////
// Returns the suffix of a file name,
// provided it is no longer than 
// the specified maximum suffix length
///////////////////////////////////////

char* GetFileSuffix (char *name)
{
    short	i, len, nameEnd = 0, suffixMaxLength = 11;
    
    len = strlen(name);
    
    for (i = 1; i < suffixMaxLength+2; i++)
    {
        if (name[len-i] == '.')
        {
            nameEnd = (len - i);
            break;
        }
    }
    
    return ((char *)&name[nameEnd]);
}


////////////////////////////////////////
// Cuts suffix from a file name
///////////////////////////////////////

char* CutSuffix (char *name)
{
    short	i, len, suffixMaxLength = 11;
    
    len = strlen(name);
    
    for (i = 1; i < suffixMaxLength+2; i++)
    {
        if (name[len-i] == '.')
        {
            name[len-i] = NULL;
            return name;
        }
    }
    return name;
}


////////////////////////////////////////
// Cuts suffix from a file name
///////////////////////////////////////

char* GetFileNameFromPath (char *name)
{
    short	i, len;
    
    len = strlen(name);
    
    for (i = len; i > 0; i--)
    {
        if (name[i] == '/')
        {
            return((char *)&name[i+1]);
        }
    }
    return name;
}





////////////////////////////////////////
// Determines whether file exists at path or not
///////////////////////////////////////

short DoesFileExist (char *path)
{
	if (access(path, F_OK) == -1)
		return false;

	return true;	
}



/*************************************
Returns path to a file in Resources folder 
within bundle, when provided a name
*************************************/
OSErr GetResourcePath (char *name, char *filePath)
{
	OSErr			err = noErr;
	CFBundleRef 	appBundle;
	CFStringRef 	fileName;
	CFURLRef		resourceFileURL;
	FSRef			fileRef;
	FSSpec			fileSpec;
	
	//make sure we have a valid name
	/*if (strlen(name) < 1 || name == NULL)
		return true;*/
		
	//get app bundle
	appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return true;
	
	//convert file name that was passed to us into a CFString
	fileName = CFStringCreateWithCString( NULL, name, kCFStringEncodingASCII );
    if (fileName == NULL)
        return true;
	
	//get url of resources folder
	resourceFileURL = CFBundleCopyResourceURL(appBundle, fileName, NULL, NULL);

	//Get file reference from Core Foundation URL
    if ( !CFURLGetFSRef( resourceFileURL, &fileRef ))
        return true;
    
    CFRelease(resourceFileURL);
    CFRelease(fileName);
    
    //convert FSRef to FSSpec
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return err;
       
	//convert fsspec to path string
    err = FSMakePath(fileSpec, (unsigned char *)filePath, kMaxPathLength);
    if (err)
        return err;
    
    //make sure it exists :)
    if (!DoesFileExist(filePath))
        return true;
   	
   	return noErr;
}