/*
    Platypus - create MacOS X application bundles that execute scripts
        This is the executable that goes into Platypus apps
    Copyright (C) 2003 Sveinbjorn Thordarson <sveinbt@hi.is>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    main.c - main program file

*/

////////////////////////////////////////
// Includes
///////////////////////////////////////    
#pragma mark Includes

// Apple stuff
#include <Carbon/Carbon.h>
#include <CoreFoundation/CoreFoundation.h>
#include <Security/Authorization.h>

// Unix stuff
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include <pthread.h>

////////////////////////////////////////
// Definitions
///////////////////////////////////////    
#pragma mark Definitions

// name length limits
#define	kMaxInterpreterLength	256

#define	kMaxPathLength	1024

// execution modes
#define	kErrGettingExecStyle	0
#define	kNormalExecution	1
#define	kPrivilegedExecution	2

// output modes
#define	kErrGettingOutputType	0
#define	kNoOutput		1
#define	kProgressBarOutput	2
#define	kTextOutput		3

//exec style strings
#define kExecStyleNoAuth	"NoAuth"
#define kExecStyleAuth		"Auth"

//output type strings
#define	kOutputTypeNone		"None"
#define	kOutputTypeProgressBar	"ProgressBar"
#define	kOutputTypeText		"Text"

// names of files bundled with app
#define	kScriptFileName		"script"
#define	kInterpreterFileName	"interpreter"
#define	kExecStyleFileName	"auth"
#define	kOutputTypeFileName	"outputType"
    
//maximum arguments the script accepts 
#define	kMaxArgumentsToScript	252

////////////////////////////////////////
// Prototypes
///////////////////////////////////////    
#pragma mark Prototypes

static void *Execute (void *arg);
static OSErr ExecuteScript (char *interpreter, char *script);
static OSErr ExecuteScriptWithPrivileges (char *interpreter, char *script);

static void  GetParameters (void);
static char* GetScript (void);
static char* GetInterpreter (void);
static short GetExecutionStyle (void);
static short GetOutputType (void);

OSErr LoadProgressWindow (void);
OSErr LoadMenuBar (char *appName);

static OSStatus FSMakePath(FSSpec file, char *path, long maxPathSize);
static void RedFatalAlert (Str255 errorString, Str255 expStr);
static short DoesFileExist (char *path);
static char* GetFileNameFromPath (char *name);

void QuitApp (void);

static OSErr AppQuitAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon);
static OSErr AppOpenDocAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon);
static OSErr AppOpenAppAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon);

////////////////////////////////////////
// globals
///////////////////////////////////////    
#pragma mark Globals

// process id of forked process that executes script
    int		pid = 0;
// thread id of thread that starts running script
    pthread_t	tid = 0;
    
// indicator of whether the script has completed executing
    short	taskDone = true;

    // execution parameters
    char		interpreter[kMaxPathLength];
    char		scriptPath[kMaxPathLength];
    short		execStyle = 0;
    short		outputType = 0;

//arguments to the script
char		*arguments[kMaxArgumentsToScript+3];
char		*fileArgs[kMaxArgumentsToScript];
short		numArgs = 0;

extern char	**environ;

// controls, Mac API
WindowRef	theWindow;
ControlRef	textField;

#pragma mark -

////////////////////////////////////////
// Program entrance point
///////////////////////////////////////
int main(int argc, char* argv[])
{
    OSErr			err = noErr;

    InitCursor();

    //install Apple Event handlers
    err += AEInstallEventHandler( kCoreEventClass, kAEQuitApplication, NewAEEventHandlerUPP((AEEventHandlerProcPtr)AppQuitAEHandler), 0, false );
    err += AEInstallEventHandler( kCoreEventClass, kAEOpenDocuments, NewAEEventHandlerUPP((AEEventHandlerProcPtr)AppOpenDocAEHandler), 0, false );
    err += AEInstallEventHandler( kCoreEventClass, kAEOpenApplication, NewAEEventHandlerUPP((AEEventHandlerProcPtr)AppOpenAppAEHandler), 0, false );
    if (err != noErr)
        RedFatalAlert("\pInitialization Error", "\pError initing Apple Event handlers.");

    //create the menu bar
    err = LoadMenuBar(NULL);
    if (err != noErr)
        RedFatalAlert("\pInitialization Error", "\pError loading Menu Bar from Nib.");
    
    //load data from files containing exec settings
    GetParameters();
        
    //at this point, we've loaded all the requisite information into globals
    //now we want to make sure the nib files containing windows and stuff like that are present
    //and then get the data they contain
    if (outputType == kProgressBarOutput)
    {
        err = LoadProgressWindow();
        if (err != noErr)
            RedFatalAlert("\pInitialization Error", "\pError loading Progress Bar Window from Nib.");
    }
    
    /**** NOT IMPLEMENTED ****/
    if (outputType == kTextOutput)
    {
        //err = LoadTextWindow();
        //if (err != noErr)
        //    RedFatalAlert("\pInitialization Error", "\pError loading Text Window from Nib.");
        RedFatalAlert("\pInitialization Error", "\pUnimplemented output mode set in app config files.");
    }

    //Run the event loop
    RunApplicationEventLoop();
    return 0;
}

#pragma mark -


///////////////////////////////////
// Execution thread starts here
// Determine execution style and
// run the bundled script.
///////////////////////////////////
static void *Execute (void *arg)
{
    OSErr		err = noErr;

    //ok, we've starting executing task and we're not finished
    taskDone = false;
    
    // Execute script
    switch(execStyle)
    {
        //don't need administrator privileges
        case kNormalExecution:
        {
            //execute by forking and execing
            err = ExecuteScript(interpreter, scriptPath);
            
        } break;
        
        //we need to authenticate - script runs as root
        case kPrivilegedExecution:
        {
            //authenticate and run
            err = ExecuteScriptWithPrivileges (interpreter, scriptPath);
            
        } break;
    }
    
    
    taskDone = true;

    ExitToShell();
    return 0;
}



////////////////////////////////////////
// Run script via the system command
///////////////////////////////////////
static OSErr ExecuteScript (char *interpreter, char *script)
{
    char	errStr[256];
    Str255	pErrStr;
    pid_t	wpid;
    int		status, i;
    
    /*******************************************************************
    // Generate the array of argument strings before we do any executing
    ********************************************************************/
    
            //first two argument to execve are always the app name and then the script to be executed
            arguments[0] = (char *)GetFileNameFromPath(interpreter);
            arguments[1] = scriptPath;
            
            //if we have files as arguments to script, add them
            if (numArgs > 0)
            {
                for (i = 0; i < numArgs; i++)
                {
                    arguments[i+2] = fileArgs[i];
                }
                arguments[i+3] = NULL;
            }
            else //otherwise terminate string list
                arguments[2] = NULL;
    
    //open fork
    pid = fork();
    
    if (pid == (pid_t)-1)
    {
        //error
        sprintf((char *)&errStr, "Forking failed.  fork() returned %d", pid);
        CopyCStringToPascal ((char *)&errStr, pErrStr);
        RedFatalAlert("\pExecution Error", pErrStr);
        exit(13);
    }
    else if (pid == 0)
    {
        //child process started
        //execute the script
        execve(interpreter, arguments, environ);
        
        //if we reach this point there's an error
        sprintf((char *)&errStr, "%s %s execve failed.", interpreter, arguments[1]);
        CopyCStringToPascal ((char *)&errStr, pErrStr);
        RedFatalAlert("\pExecution Error", pErrStr);
    }

    //wait while child process finishes
    wpid = wait(&status);
    
    
    if (wpid == (pid_t)-1)
    {	
        //ERROR
        return wpid;  
    }
    
    return noErr;
}

////////////////////////////////////////
// Authenticate and run script as root
///////////////////////////////////////
static OSErr ExecuteScriptWithPrivileges (char *interpreter, char *script)
{
    OSErr			err = noErr;
    AuthorizationRef 		authorizationRef;
    short			i;
    
    //first argument to apple's function is the script name and then the script to be executed
            arguments[0] = scriptPath;
            
            //if we have files as arguments to script, add them
            if (numArgs > 0)
            {
                for (i = 0; i < numArgs; i++)
                {
                    arguments[i+1] = fileArgs[i];
                }
                arguments[i+2] = NULL;
            }
            else //otherwise terminate string list
                arguments[1] = NULL;
    
    // Use Apple's Authentication Manager APIs to run script with root privileges
    err = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, &authorizationRef);
    if (err != errAuthorizationSuccess)
        return err;

    if ((err = AuthorizationExecuteWithPrivileges(authorizationRef, interpreter, kAuthorizationFlagDefaults, arguments, NULL)) != noErr)
    {
        //some kind of error took place -- most likely, authentication failed
        return true;
    }
    else
    {
            //wait for child process to finish
            int child;
            wait(&child);
    }

    return noErr;
}



#pragma mark -

////////////////////////////////////////
// This function loads all the neccesary settings
// from config files in the Resources folder
///////////////////////////////////////
static void GetParameters (void)
{
    char *str;

    // Make sure that 'interpreter', 'execStyle', 'outputType'
    // and 'script' files are present and get their data
    
    //get path to interpreter
    str = (char *)GetInterpreter();
    if (str == NULL)
        RedFatalAlert("\pInitialization Error", "\pError getting interpreter from application package.");
    else
        strcpy((char *)&interpreter, str);
    
    //get path to script to be executed
    str = (char *)GetScript();
    if (str == NULL)
        RedFatalAlert("\pInitialization Error", "\pError getting script from application package.");
    else
        strcpy((char *)&scriptPath, str);
    
    //check if we need root privileges
    execStyle = GetExecutionStyle();
    if (execStyle == kErrGettingExecStyle)
        RedFatalAlert("\pInitialization Error", "\pError determining execution style.");
    
    //check if we're doing some kind of output
    outputType = GetOutputType();
    if (outputType == kErrGettingOutputType)
        RedFatalAlert("\pInitialization Error", "\pError determining execution style.");


}


////////////////////////////////////////
// Get path to the script in Resources folder
///////////////////////////////////////
static char* GetScript (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		scriptFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    char		*path;
    OSErr		err = noErr;

    //get CF URL for interpreter text file
    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, kScriptFileName, kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    scriptFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (scriptFileURL == NULL)
        return NULL;
    
    //Get file reference from Core Foundation URL
    if ( !CFURLGetFSRef( scriptFileURL, &fileRef ))
        return NULL;
    
    //dispose of the CF variables
    CFRelease( scriptFileURL );
    CFRelease( fileName );
    
    //convert FSRef to FSSpec
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
        
    //create path string
    path = malloc(kMaxPathLength);
    if (path == NULL)
        return NULL;
    
    //convert fsspec to path string
    err = FSMakePath(fileSpec, path, kMaxPathLength);
    if (err)
        return NULL;
    
    //make sure it exists :)
    if (!DoesFileExist(path))
        return NULL;
    
    return (path);

}

////////////////////////////////////////
// Gets the path to interpreter from
// text file in resources folder
///////////////////////////////////////
static char* GetInterpreter (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		interpreterFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    static char 	interpreter[kMaxInterpreterLength];
    char		*path;
    FILE		*file;
    OSErr		err;
    
    //get CF URL for interpreter text file
    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, kInterpreterFileName, kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    interpreterFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (interpreterFileURL == NULL)
        return NULL;
    
    //Get file reference from Core Foundation URL
    if ( !CFURLGetFSRef( interpreterFileURL, &fileRef ))
        return NULL;
    
    //dispose of the CF variables
    CFRelease( interpreterFileURL );
    CFRelease( fileName );
        
    //convert FSRef to FSSpec
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
    
    //create path string
    path = malloc(kMaxPathLength);
    if (path == NULL)
        return NULL;
    
    //convert fsspec to path string
    err = FSMakePath(fileSpec, path, kMaxPathLength);
    if (err)
        return NULL;
    
    //make sure text file containing interpreter exists
    if (!DoesFileExist(path))
        return NULL;
    
    //open file, read interpreter from first line, close
    file = fopen(path, "r");
    fgets(interpreter, kMaxInterpreterLength, file);
    fclose(file);
    
    //free memory containing path name
    free(path);

    //make sure interpreter exists
    if (!DoesFileExist((char *)&interpreter))
        RedFatalAlert("\pExecution Error", "\pThe interpreter for the script this application executes could not be found.");
    
    return ((char *)&interpreter);
}

////////////////////////////////////////
// Open text file in Resources folder
// which tells whether the script needs
// to run as root or not
///////////////////////////////////////
static short GetExecutionStyle (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		execStyleFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    char 		str[7];
    char		*path;
    FILE		*file;
    OSErr		err;
    
    //get CF URL for interpreter text file
    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, kExecStyleFileName, kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    execStyleFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (execStyleFileURL == NULL)
        return NULL;
    
    //Get file reference from Core Foundation URL
    if ( !CFURLGetFSRef( execStyleFileURL, &fileRef ))
        return NULL;
    
    //dispose of the CF variables
    CFRelease( execStyleFileURL );
    CFRelease( fileName );
        
    //convert FSRef to FSSpec
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
    
    //create path string
    path = malloc(kMaxPathLength);
    if (path == NULL)
        return NULL; 
        
    //convert fsspec to path string
    err = FSMakePath(fileSpec, path, kMaxPathLength);
    if (err)
        return NULL;
    
    //make sure text file containing interpreter exists
    if (!DoesFileExist(path))
        return NULL;
    
    //open file, read exec style from first line, close
    file = fopen(path, "r");
    fgets(str,7,file);
    fclose(file);
    
    //free memory containing path string
    free(path);

    //return style based on what the exec style text file contains
    if (!strcmp((char *)&str, (char *)&kExecStyleNoAuth))
        return kNormalExecution;
    
    if (!strcmp((char *)&str, (char *)&kExecStyleAuth))
        return kPrivilegedExecution;
    
    return kNormalExecution;
}


////////////////////////////////////////
// Open text file in Resources folder that
// contains output type, i.e. progress bar, text etc.
///////////////////////////////////////
static short GetOutputType (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		outputTypeFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    char 		str[12];
    char		*path;
    FILE		*file;
    OSErr		err;
    
    //get CF URL for interpreter text file
    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, kOutputTypeFileName, kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    outputTypeFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (outputTypeFileURL == NULL)
        return NULL;
    
    //Get file reference from Core Foundation URL
    if ( !CFURLGetFSRef( outputTypeFileURL, &fileRef ))
        return NULL;
    
    //dispose of the CF variables
    CFRelease( outputTypeFileURL );
    CFRelease( fileName );
        
    //convert FSRef to FSSpec
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
    
    //create path string
    path = malloc(kMaxPathLength);
    if (path == NULL)
        return NULL; 
        
    //convert fsspec to path string
    err = FSMakePath(fileSpec, path, kMaxPathLength);
    if (err)
        return NULL;
    
    //make sure text file containing interpreter exists
    if (!DoesFileExist(path))
        return NULL;
    
    //open file, read exec style from first line, close
    file = fopen(path, "r");
    fgets(str,sizeof(str),file);
    fclose(file);
    
    //free memory containing path string
    free(path);

    //return style based on what the exec style text file contains
    if (!strcmp((char *)&str, (char *)&kOutputTypeNone))
        return kNoOutput;
    
    if (!strcmp((char *)&str, (char *)&kOutputTypeProgressBar))
        return kProgressBarOutput;
    
    if (!strcmp((char *)&str, (char *)&kOutputTypeText))
        return kTextOutput;
    
    return kNoOutput;
}


#pragma mark -

//////////////////////////////////////
//  Load progress bar
/////////////////////////////////////

OSErr LoadProgressWindow (void)
{
    OSErr		err;
    IBNibRef 		nibRef;
    
    err = CreateNibReference(CFSTR("ProgressWindow"), &nibRef);
    if (err)
        return err;
        
    err = CreateWindowFromNib(nibRef, CFSTR("Window"), &theWindow);
    if (err)
        return err;

    DisposeNibReference(nibRef);
    
    // The window was created hidden so show it.
    ShowWindow( theWindow );

    return noErr;
}

//////////////////////////////////////
//  Load window for text output
/////////////////////////////////////

/*OSErr LoadTextWindow (void)
{
    OSErr		err;
    IBNibRef 		nibRef;
    ControlID		controlID;
    //Str255		pName;
    
    err = CreateNibReference(CFSTR("TextWindow"), &nibRef);
    if (err)
        return err;
        
    err = CreateWindowFromNib(nibRef, CFSTR("Window"), &theWindow);
    if (err)
        return err;

    DisposeNibReference(nibRef);

    //app name text field
    controlID.signature = 'Text';
    controlID.id = 0;
    GetControlByID (theWindow, &controlID, &textField);
    
    //set window title
   // CopyCStringToPascal (appName, pName);
    //SetWTitle (theWindow, pName);
    
    // The window was created hidden so show it.
    ShowWindow( theWindow );

    return noErr;
}
*/
//////////////////////////////////////
//  Load menu bar from nib
/////////////////////////////////////

OSErr LoadMenuBar (char *appName)
{
    OSErr		err;
    IBNibRef 		nibRef;
    
    err = CreateNibReference(CFSTR("MenuBar"), &nibRef);
    if (err)
        return err;
    
    err = SetMenuBarFromNib(nibRef, CFSTR("MenuBar"));
    if (err)
        return err;

    DisposeNibReference(nibRef);

    return noErr;
}



#pragma mark -

////////////////////////////////////////
// Generate path string from FSSpec record
///////////////////////////////////////
static OSStatus FSMakePath(FSSpec file, char *path, long maxPathSize)
{
    OSErr 	err = noErr;
    FSRef  	fileRef;

    //create file reference from file spec
    err = FSpMakeFSRef(&file, &fileRef);
    if (err)
        return err;

    /* and then convert the FSRef to a path */
    err = FSRefMakePath(&fileRef, path, maxPathSize);
    return (err);
}



////////////////////////////////////////
//Standard red error alert, then exit application
////////////////////////////////////////
static void RedFatalAlert (Str255 errorString, Str255 expStr)
{
    StandardAlert (kAlertStopAlert, errorString,  expStr, NULL, NULL);
    ExitToShell();
}

////////////////////////////////////////
// Determines whether file exists at path or not
///////////////////////////////////////

static short DoesFileExist (char *path)
{
    if (access(path, F_OK) == -1)
            return false;

    return true;	
}

////////////////////////////////////////
// Cuts suffix from a file name
///////////////////////////////////////

static char* GetFileNameFromPath (char *name)
{
    short	i, len;
    
    len = strlen(name);
    
    for (i = len; i > 0; i--)
    {
        if (name[i] == '/')
        {
            return((char *)&name[i+1]);
        }
    }
    return name;
}

#pragma mark -



////////////////////////////////////////
// Do needed stuff and then exit app
///////////////////////////////////////
void QuitApp (void)
{
    short	i;

    // This function could be called theoretically at any time during the execution of the script thread
    // so we must deallocate memory reserved for argument strings
    if (numArgs)
    {
        for (i = 0; i < numArgs; i++)
        {
            if (fileArgs[i] != NULL)
            {
                 free(fileArgs[i]);
            }
        }
    }
    
    //we need to kill the task if it's still running
    if (taskDone != true)
    {
        if (pid != 0)
        {
            //kill the script process brutally
            kill(pid, 9);
            printf("Platypus App: PID %d killed brutally\n", pid);
        }
    }
    //kill the thread that ran the script
    pthread_cancel(tid);
    ExitToShell();
}

#pragma mark -

////////////////////////////////////////
// Apple Event handler for Quit i.e. from
// the dock or Application menu item
///////////////////////////////////////
static OSErr AppQuitAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon)
{
	#pragma unused (reply, refCon, theAppleEvent)
	
  	QuitApp();
  	
  	return noErr;
}

/*************************************
Handler for docs dragged on app icon
*************************************/

static OSErr AppOpenDocAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon)
{
	#pragma unused (reply, refCon)
	
        OSErr		err = noErr;
        AEDescList	fileSpecList;
        AEKeyword	keyword;
        DescType	type;
        
        short		i;
        long		count;
        long		actualSize;
        int		ret;
        
        FSSpec		fileSpec;
        char		path[kMaxPathLength];
        
        // make sure it isn't possible to drag addition items on to the app once it's processing
        // the batch that was dropped on to it at startup
        if (!taskDone)
        {
            return noErr;
        }
        
        //Read the AppleEvent
        err = AEGetParamDesc(theAppleEvent, keyDirectObject, typeAEList, &fileSpecList);
		
        //Count number of files
        err = AECountItems(&fileSpecList, &count);
                
        //iteratively process each file
        // In this case, we get the path of each file dragged on application and put them into the fileArgs string array
        for (i = 1; i <= count; i++) 
        {
                //get fsspec from apple event
                err = AEGetNthPtr(&fileSpecList, i, typeFSS, &keyword, &type, (Ptr)&fileSpec, sizeof(FSSpec), &actualSize);
                if (err == noErr) 
                {
                            //get path from file spec
		            err = FSMakePath(fileSpec, (unsigned char *)&path, kMaxPathLength);
		            if (err)
		            	return err;
                            
                            //make sure we don't add more files as argument than we can handle
                            if (numArgs == kMaxArgumentsToScript)
                            {
                                break;
                            }
                            
                            //allocate memory for path string in fileArgs array
                            fileArgs[numArgs] = malloc(kMaxPathLength);
                            
                            //check if memory allocation failed
                            if (fileArgs[numArgs] == NULL)
                            {
                                return true;
                            }	
                            //copy path into fileArgs array
                            strcpy(fileArgs[numArgs], (char *)&path);
                            numArgs++;  
                }
                else
                    return err;
        }
        
        //start new thread that executes the script now that we have all the required file arguments
        ret = pthread_create(&tid, NULL, Execute, NULL);
        
        return err;
}

static OSErr AppOpenAppAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon)
{
	#pragma unused (reply, refCon, theAppleEvent)
	
        int	ret;
        
        // Since we're running here, it means the app has been opened without any items dragged on to it
  	ret = pthread_create(&tid, NULL, Execute, NULL);
  	return noErr;
}

